from abc import abstractmethod

from torch.utils import data

from yqn_config.base_config import BaseConfig


class BaseDataset(data.Dataset):
    def __init__(self, config: BaseConfig):
        self.config = config
        super(BaseDataset, self).__init__()

    @abstractmethod
    def get_item_size(self):
        pass

    @abstractmethod
    def get_item(self, index):
        pass

    def __len__(self):
        return self.get_item_size() * self.config.repeat_count

    def __getitem__(self, index):
        real_index = index % (self.get_item_size())
        data = self.get_item(real_index)
        return data



import argparse
import os
import pathlib
import datetime
from string import Template

import sys

APP_DEC = """                                                                           
YYYYYYY       YYYYYYY          QQQQQQQQQ           NNNNNNNN        NNNNNNNN
Y:::::Y       Y:::::Y        QQ:::::::::QQ         N:::::::N       N::::::N
Y:::::Y       Y:::::Y      QQ:::::::::::::QQ       N::::::::N      N::::::N
Y::::::Y     Y::::::Y     Q:::::::QQQ:::::::Q      N:::::::::N     N::::::N
YYY:::::Y   Y:::::YYY     Q::::::O   Q::::::Q      N::::::::::N    N::::::N
   Y:::::Y Y:::::Y        Q:::::O     Q:::::Q      N:::::::::::N   N::::::N
    Y:::::Y:::::Y         Q:::::O     Q:::::Q      N:::::::N::::N  N::::::N
     Y:::::::::Y          Q:::::O     Q:::::Q      N::::::N N::::N N::::::N
      Y:::::::Y           Q:::::O     Q:::::Q      N::::::N  N::::N:::::::N
       Y:::::Y            Q:::::O     Q:::::Q      N::::::N   N:::::::::::N
       Y:::::Y            Q:::::O  QQQQ:::::Q      N::::::N    N::::::::::N
       Y:::::Y            Q::::::O Q::::::::Q      N::::::N     N:::::::::N
       Y:::::Y            Q:::::::QQ::::::::Q      N::::::N      N::::::::N
    YYYY:::::YYYY          QQ::::::::::::::Q       N::::::N       N:::::::N
    Y:::::::::::Y            QQ:::::::::::Q        N::::::N        N::::::N
    YYYYYYYYYYYYY              QQQQQQQQ::::QQ      NNNNNNNN         NNNNNNN
                                       Q:::::Q                             
                                        QQQQQQ                             
                                                                           
"""

templates = {
    'config': [
        'template_config$cfg',
        'template_config$py',
    ],
    'infer': [
        'template_infer$py',
    ],
    'model': [
        'template_model$py',
    ],
    'train': [
        'template_acc$py',
        'template_dataset$py',
        'template_engine$py',
        'template_loss$py',
        'template_train$py',
    ],
}


def capitalize(name):
    return name.capitalize()


def get_args_parser():
    from yqn_cli import __version__
    parser = argparse.ArgumentParser(description='create module base on yqn pytorch framework')
    parser.add_argument('-version', action='version', version='%(prog)s ' + __version__)
    parser.add_argument('-name', type=str, default='*', help='module name exp: "key_attention"')
    parser.add_argument('-out', type=str, default='./', help='file output path')
    parser.add_argument('-author', type=str, default='yqn', help='author')
    return parser


def get_name_for_class(module_name):
    return ''.join(list(map(capitalize, module_name.split('_'))))


def create_dirs(module_name, output_dir):
    pathlib.Path(os.path.join(output_dir,
                              module_name,
                              "config")).mkdir(parents=True, exist_ok=True)
    pathlib.Path(os.path.join(output_dir,
                              module_name,
                              "train")).mkdir(parents=True, exist_ok=True)
    pathlib.Path(os.path.join(output_dir,
                              module_name,
                              "model")).mkdir(parents=True, exist_ok=True)
    pathlib.Path(os.path.join(output_dir,
                              module_name,
                              "infer")).mkdir(parents=True, exist_ok=True)


def output_gen_files(args, class_name, template_dir):
    module_name, output_dir, author = args.name, args.out, args.author
    now = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
    for sub_dir in templates:
        files = templates[sub_dir]
        for file in files:
            lines = []
            template_file_path = os.path.join(template_dir,
                                              sub_dir,
                                              file)
            if sub_dir == 'config':
                out_name = file.replace("template_", module_name + "_").replace("$", ".")
            else:
                out_name = file.replace("template_",
                                        "pytorch_" + module_name + "_").replace("$", ".")
            gen_file_path = os.path.join(output_dir,
                                         module_name,
                                         sub_dir,
                                         out_name)
            template_file = open(template_file_path)
            gen_file = open(gen_file_path, "w")
            tmp = Template(template_file.read())
            lines.append(tmp.safe_substitute(class_name=class_name, module_name=module_name))
            gen_file.writelines(lines)
            template_file.close()
            gen_file.close()


def parse_command(parser_fn=get_args_parser, printed=True):
    root_project_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    # 项目次目录
    module_dir = os.path.join(root_project_dir, "yqn_cli")
    print(module_dir)
    template_dir = os.path.join(module_dir, 'template')
    args = parser_fn().parse_args()
    if printed:
        param_str = '\n'.join(['%20s = %s' % (k, v) for k, v in sorted(vars(args).items())])
        print('usage: %s\n%20s   %s\n%s\n%s\n' % (' '.join(sys.argv), 'ARG', 'VALUE', '_' * 50, param_str))
    create_dirs(args.name, args.out)
    class_name = get_name_for_class(args.name)
    output_gen_files(args, class_name, template_dir)


def main():
    print(APP_DEC)
    parse_command()


if __name__ == "__main__":
    main()



import torch
from torch import nn
from torch.nn.utils.rnn import pack_padded_sequence, pad_packed_sequence

from yqn_pytorch_framework.pytorch_initialize import prepare_pack_padded_sequence, init_embedding, init_linear
from booking_ner.data.pytorch_ner_data_utils import load_word2vec
import pickle

class BiLSTM(nn.Module):
    """
        BiLSTM
    """

    def __init__(self, **kwargs):
        super(BiLSTM, self).__init__()
        for k in kwargs:
            self.__setattr__(k, kwargs[k])

        V = self.embed_num
        D = self.embed_dim
        C = self.label_num
        padding_id = self.padding_id

        self.embed = nn.Embedding(V, D, padding_idx=padding_id)
        init_embedding(self.embed.weight)

        print("self.emb_file:", self.emb_file)
        with open(self.map_file, "rb") as f:
            char_to_id, id_to_char, tag_to_id, id_to_tag = pickle.load(f)
        self.pretrained_weight = load_word2vec(self.emb_file , id_to_char, self.embed_dim, self.embed)
        self.embed = self.pretrained_weight

        self.seg_embed = nn.Embedding(5, 20, padding_idx=padding_id)
        # if False:
        #     print("pretrained_weight:::",   self.pretrained_weight)
        #     self.embed.weight.data.copy_(torch.from_numpy(self.pretrained_weight))
        # else:
        #     init_embedding(self.embed.weight)

        self.dropout_embed = nn.Dropout(self.dropout_emb)
        self.dropout = nn.Dropout(self.dropout)

        self.bilstm = nn.LSTM(input_size=D+20,
                              hidden_size=self.lstm_hiddens,
                              num_layers=self.lstm_layers,
                              bidirectional=True, batch_first=True, bias=True)

        self.linear = nn.Linear(in_features=self.lstm_hiddens * 2, out_features=C, bias=True)
        init_linear(self.linear)

    def forward(self, word, sentence_length, seg):
        """
        :param word:
        :param sentence_length:
        :return:
        """

        # print("wordwordwordword", word)
        # print("sentence_length:", sentence_length)

        # word, sentence_length, sorted_indices = prepare_pack_padded_sequence(word,
        #                                                                      sentence_length,
        #                                                                      device=self.device)
        # # print("word22222", word)
        # # print("sentence_length:", sentence_length)
        # # print("sorted_indices:", sorted_indices)
        # x = self.embed(word)  # (N,W,D)
        # seg = self.seg_embed(seg)
        # x = self.dropout_embed(x)
        # seg = self.dropout_embed(seg)
        # seg = seg[sorted_indices]
        #
        # packed_embed = pack_padded_sequence(x, sentence_length, batch_first=True, enforce_sorted=True)
        # emb_token_seg = [x, seg]
        # x = torch.cat(emb_token_seg,-1)
        # x, _ = self.bilstm(x)
        # x, _ = pad_packed_sequence(x, batch_first=True)
        # x = x[sorted_indices]
        # x = self.dropout(x)
        # x = torch.tanh(x)
        # logits = self.linear(x)



        # segs = segs.to("cuda:0")
        segs = seg
        word, sentence_length, desorted_indices = prepare_pack_padded_sequence(word, sentence_length, device=self.device)
        # segs, sentence_length_segs, desorted_indices_segs = prepare_pack_padded_sequence(segs, sentence_length,device=self.device)

        # print("word_model:", word)
        # print("sentence_length:", sentence_length)
        # print("desorted_indices:", desorted_indices)
        x = self.embed(word)  # (N,W,D)

        segs_embed = self.seg_embed(segs)
        embed_list = [x,segs_embed]
        embed_list = torch.cat(embed_list,-1)
        x = self.dropout_embed(embed_list)

        packed_embed = pack_padded_sequence(x, sentence_length, batch_first=True)
        x, _ = self.bilstm(packed_embed)
        x, _ = pad_packed_sequence(x, batch_first=True)
        x = x[desorted_indices]
        x = self.dropout(x)
        x = torch.tanh(x)
        logit = self.linear(x)
        return logit



import xml.etree.ElementTree as ET


def read_objects(file_path):
    # print(file_path)
    tree = ET.parse(file_path)
    label_items = {}
    object_items = []
    for elem in tree.iter():
        # 属性值
        if 'filename' in elem.tag:
            label_items['file_name'] = elem.text
        elif 'object' in elem.tag:
            xmin = -100
            ymin = -100
            xmax = -100
            ymax = -100
            name = None
            find = False
            for attr in list(elem):
                if 'bndbox' in attr.tag:
                    xmin = int(round(float(attr.find('xmin').text)))
                    ymin = int(round(float(attr.find('ymin').text)))
                    xmax = int(round(float(attr.find('xmax').text)))
                    ymax = int(round(float(attr.find('ymax').text)))
                    find = True
                elif 'name' in attr.tag:
                    name = attr.text
            if find:
                object_item = {'box': (xmin, ymin, xmax, ymax), 'name': name}
                object_items.append(object_item)
    label_items['object'] = object_items
    return label_items



import torch

cpu_device = "cpu"


def get_cpu_device():
    device = torch.device("cpu")
    return device


def get_device(verbose=False):
    train_on_gpu = torch.cuda.is_available()
    if verbose:
        if not train_on_gpu:
            print('CUDA is not available. Run on CPU')
        else:
            print('CUDA is available. Run on GPU')

    device = torch.device("cuda:0" if train_on_gpu else "cpu")
    return device


def tensor_load_device(torch_tensor, device):
    if device == get_cpu_device():
        torch_tensor.to(device)
    else:
        torch_tensor.cuda()
    if torch.cuda.device_count() > 1:
        if isinstance(torch_tensor, torch.nn.Module):
            torch_tensor = torch.nn.DataParallel(torch_tensor)
    return torch_tensor



import torch
import torch.nn as nn
from .utils import load_state_dict_from_url

__all__ = [
    'ShuffleNetV2', 'shufflenet_v2_x0_5', 'shufflenet_v2_x1_0',
    'shufflenet_v2_x1_5', 'shufflenet_v2_x2_0'
]

model_urls = {
    'shufflenetv2_x0.5': 'https://download.pytorch.org/models/shufflenetv2_x0.5-f707e7126e.pth',
    'shufflenetv2_x1.0': 'https://download.pytorch.org/models/shufflenetv2_x1-5666bf0f80.pth',
    'shufflenetv2_x1.5': None,
    'shufflenetv2_x2.0': None,
}


def channel_shuffle(x, groups):
    # type: (torch.Tensor, int) -> torch.Tensor
    batchsize, num_channels, height, width = x.data.size()
    channels_per_group = num_channels // groups

    # reshape
    x = x.view(batchsize, groups,
               channels_per_group, height, width)

    x = torch.transpose(x, 1, 2).contiguous()

    # flatten
    x = x.view(batchsize, -1, height, width)

    return x


class InvertedResidual(nn.Module):
    def __init__(self, inp, oup, stride):
        super(InvertedResidual, self).__init__()

        if not (1 <= stride <= 3):
            raise ValueError('illegal stride value')
        self.stride = stride

        branch_features = oup // 2
        assert (self.stride != 1) or (inp == branch_features << 1)

        if self.stride > 1:
            self.branch1 = nn.Sequential(
                self.depthwise_conv(inp, inp, kernel_size=3, stride=self.stride, padding=1),
                nn.BatchNorm2d(inp),
                nn.Conv2d(inp, branch_features, kernel_size=1, stride=1, padding=0, bias=False),
                nn.BatchNorm2d(branch_features),
                nn.ReLU(inplace=True),
            )
        else:
            self.branch1 = nn.Sequential()

        self.branch2 = nn.Sequential(
            nn.Conv2d(inp if (self.stride > 1) else branch_features,
                      branch_features, kernel_size=1, stride=1, padding=0, bias=False),
            nn.BatchNorm2d(branch_features),
            nn.ReLU(inplace=True),
            self.depthwise_conv(branch_features, branch_features, kernel_size=3, stride=self.stride, padding=1),
            nn.BatchNorm2d(branch_features),
            nn.Conv2d(branch_features, branch_features, kernel_size=1, stride=1, padding=0, bias=False),
            nn.BatchNorm2d(branch_features),
            nn.ReLU(inplace=True),
        )

    @staticmethod
    def depthwise_conv(i, o, kernel_size, stride=1, padding=0, bias=False):
        return nn.Conv2d(i, o, kernel_size, stride, padding, bias=bias, groups=i)

    def forward(self, x):
        if self.stride == 1:
            x1, x2 = x.chunk(2, dim=1)
            out = torch.cat((x1, self.branch2(x2)), dim=1)
        else:
            out = torch.cat((self.branch1(x), self.branch2(x)), dim=1)

        out = channel_shuffle(out, 2)

        return out


class ShuffleNetV2(nn.Module):
    def __init__(self, stages_repeats, stages_out_channels, num_classes=1000, inverted_residual=InvertedResidual):
        super(ShuffleNetV2, self).__init__()

        if len(stages_repeats) != 3:
            raise ValueError('expected stages_repeats as list of 3 positive ints')
        if len(stages_out_channels) != 5:
            raise ValueError('expected stages_out_channels as list of 5 positive ints')
        self._stage_out_channels = stages_out_channels

        input_channels = 3
        output_channels = self._stage_out_channels[0]
        self.conv1 = nn.Sequential(
            nn.Conv2d(input_channels, output_channels, 3, 2, 1, bias=False),
            nn.BatchNorm2d(output_channels),
            nn.ReLU(inplace=True),
        )
        input_channels = output_channels

        self.maxpool = nn.MaxPool2d(kernel_size=3, stride=2, padding=1)

        stage_names = ['stage{}'.format(i) for i in [2, 3, 4]]
        for name, repeats, output_channels in zip(
                stage_names, stages_repeats, self._stage_out_channels[1:]):
            seq = [inverted_residual(input_channels, output_channels, 2)]
            for i in range(repeats - 1):
                seq.append(inverted_residual(output_channels, output_channels, 1))
            setattr(self, name, nn.Sequential(*seq))
            input_channels = output_channels

        output_channels = self._stage_out_channels[-1]
        self.conv5 = nn.Sequential(
            nn.Conv2d(input_channels, output_channels, 1, 1, 0, bias=False),
            nn.BatchNorm2d(output_channels),
            nn.ReLU(inplace=True),
        )

        self.fc = nn.Linear(output_channels, num_classes)

    def _forward(self, x):
        x = self.conv1(x)
        x = self.maxpool(x)
        x = self.stage2(x)
        x = self.stage3(x)
        x = self.stage4(x)
        x = self.conv5(x)
        x = x.mean([2, 3])  # globalpool
        x = self.fc(x)
        return x

    forward = _forward


def _shufflenetv2(arch, pretrained, progress, *args, **kwargs):
    model = ShuffleNetV2(*args, **kwargs)

    if pretrained:
        model_url = model_urls[arch]
        if model_url is None:
            raise NotImplementedError('pretrained {} is not supported as of now'.format(arch))
        else:
            state_dict = load_state_dict_from_url(model_url, progress=progress)
            model.load_state_dict(state_dict)

    return model


def shufflenet_v2_x0_5(pretrained=False, progress=True, **kwargs):
    """
    Constructs a ShuffleNetV2 with 0.5x output channels, as described in
    `"ShuffleNet V2: Practical Guidelines for Efficient CNN Architecture Design"
    <https://arxiv.org/abs/1807.11164>`_.

    Args:
        pretrained (bool): If True, returns a model pre-trained on ImageNet
        progress (bool): If True, displays a progress bar of the download to stderr
    """
    return _shufflenetv2('shufflenetv2_x0.5', pretrained, progress,
                         [4, 8, 4], [24, 48, 96, 192, 1024], **kwargs)


def shufflenet_v2_x1_0(pretrained=False, progress=True, **kwargs):
    """
    Constructs a ShuffleNetV2 with 1.0x output channels, as described in
    `"ShuffleNet V2: Practical Guidelines for Efficient CNN Architecture Design"
    <https://arxiv.org/abs/1807.11164>`_.

    Args:
        pretrained (bool): If True, returns a model pre-trained on ImageNet
        progress (bool): If True, displays a progress bar of the download to stderr
    """
    return _shufflenetv2('shufflenetv2_x1.0', pretrained, progress,
                         [4, 8, 4], [24, 116, 232, 464, 1024], **kwargs)


def shufflenet_v2_x1_5(pretrained=False, progress=True, **kwargs):
    """
    Constructs a ShuffleNetV2 with 1.5x output channels, as described in
    `"ShuffleNet V2: Practical Guidelines for Efficient CNN Architecture Design"
    <https://arxiv.org/abs/1807.11164>`_.

    Args:
        pretrained (bool): If True, returns a model pre-trained on ImageNet
        progress (bool): If True, displays a progress bar of the download to stderr
    """
    return _shufflenetv2('shufflenetv2_x1.5', pretrained, progress,
                         [4, 8, 4], [24, 176, 352, 704, 1024], **kwargs)


def shufflenet_v2_x2_0(pretrained=False, progress=True, **kwargs):
    """
    Constructs a ShuffleNetV2 with 2.0x output channels, as described in
    `"ShuffleNet V2: Practical Guidelines for Efficient CNN Architecture Design"
    <https://arxiv.org/abs/1807.11164>`_.

    Args:
        pretrained (bool): If True, returns a model pre-trained on ImageNet
        progress (bool): If True, displays a progress bar of the download to stderr
    """
    return _shufflenetv2('shufflenetv2_x2.0', pretrained, progress,
                         [4, 8, 4], [24, 244, 488, 976, 2048], **kwargs)



import json
import os
import pathlib
import re
from abc import abstractmethod
from datetime import datetime

import yaml

from yqn_config.conf_parser import CommonConf
from yqn_exception.exception import *
from yqn_pytorch_framework.device import get_device

BOOLEAN_STATES = {'yes': True, 'true': True, 'on': True,
                  'no': False, 'false': False, 'off': False}
REQUIRE_CONFIG = ["config_name",
                  "device",
                  "repeat_count",
                  "shuffle_batch",
                  "num_epochs",
                  "epochs_shuffle",
                  "batch_size",
                  "optimizer",
                  "learning_rate",
                  "retrain",
                  "write_image",
                  "write_scalar",
                  "summary_freq",
                  "save_freq",
                  "progress_freq",
                  "split_date", ]


class BaseConfig:

    def __init__(self, project_path,
                 resource_path,
                 output_path,
                 switch_flag):
        self.project_path = project_path
        self.resource_path = resource_path
        self.output_path = output_path
        self.switch_flag = switch_flag
        self.load_config()
        super(BaseConfig, self).__init__()
        self.device = get_device()
        self.modify_config()
        self.check_config()
        self.make_dirs()

    @abstractmethod
    def get_config_file(self):
        pass

    def modify_config(self):
        pass

    def check_config(self):
        for name in REQUIRE_CONFIG:
            try:
                attr = getattr(self, name)
            except AttributeError:
                raise ConfigMissException("配置文件必须包含:", name)

    def load_config(self):
        config_parser = CommonConf()
        assert os.path.exists(self.get_config_file())
        config_parser.read(self.get_config_file())
        loader = yaml.FullLoader
        loader.add_implicit_resolver(
            u'tag:yaml.org,2002:float',
            re.compile(u'''^(?:
             [-+]?(?:[0-9][0-9_]*)\\.[0-9_]*(?:[eE][-+]?[0-9]+)?
            |[-+]?(?:[0-9][0-9_]*)(?:[eE][-+]?[0-9]+)
            |\\.[0-9_]+(?:[eE][-+][0-9]+)?
            |[-+]?[0-9][0-9_]*(?::[0-5]?[0-9])+\\.[0-9_]*
            |[-+]?\\.(?:inf|Inf|INF)
            |\\.(?:nan|NaN|NAN))$''', re.X),
            list(u'-+0123456789.'))

        lines = []
        lines.append("*" * 150)
        lines.append(" {:<40} Config File Path {:<90} ".format("", self.get_config_file()))
        for section in config_parser.sections():
            lines.append("=" * 150)
            lines.append(" Section {:<140} ".format("[" + str(section) + "]"))
            lines.append("=" * 150)
            for k, v in config_parser.items(section):
                if v.lower() in BOOLEAN_STATES:
                    v = BOOLEAN_STATES[v.lower()]
                else:
                    v = yaml.load(v, Loader=loader)
                setattr(self, k, v)
                lines.append(" {:<40} | {:<105} ".format(k, json.dumps(v)))
        # lines.append("=" * 150)
        # print("|" + "|\n|".join(lines) + "|\n")

    def make_dirs(self):
        model_dir = os.path.join(self.resource_path, "models", self.config_name)
        files_dir = os.path.join(self.resource_path, "files", self.config_name)
        train_model_out_dir = os.path.join(self.output_path, "train_model", self.config_name)
        train_log_dir = os.path.join(self.output_path, "train_log", self.config_name)
        pathlib.Path(model_dir).mkdir(parents=True, exist_ok=True)
        pathlib.Path(files_dir).mkdir(parents=True, exist_ok=True)
        pathlib.Path(train_model_out_dir).mkdir(parents=True, exist_ok=True)
        pathlib.Path(train_log_dir).mkdir(parents=True, exist_ok=True)

    def get_infer_model_path(self, model_name):
        """
            获取推断模型文件路径 {resource_path}/models/{config_name}/model_name
            #:param model_name 模型文件名
        """
        pathlib.Path(os.path.join(self.resource_path, "models", self.config_name)).mkdir(parents=True, exist_ok=True)
        model_path = os.path.join(self.resource_path, "models", self.config_name, model_name)
        return model_path

    def get_file_path(self, file_name):
        """
            获取资源文件路径 如字典文件等 {resource_path}/files/{config_name}/file_name
            #:param file_name 资源文件名
        """
        pathlib.Path(os.path.join(self.resource_path,
                                  "files",
                                  self.config_name)).mkdir(parents=True, exist_ok=True)
        file_path = os.path.join(self.resource_path,
                                 "files",
                                 self.config_name,
                                 file_name)
        return file_path

    def get_tmp_file_dir(self, split_date=False):
        """
            获取零时文件路径  {output_path}/tmp_file/{config_name}
        """
        pathlib.Path(os.path.join(self.output_path, "tmp_file", self.config_name)).mkdir(parents=True, exist_ok=True)
        if split_date:
            timestamp = "{0:%Y_%m_%d/}".format(datetime.now())
            pathlib.Path(os.path.join(self.output_path,
                                      "tmp_file",
                                      self.config_name,
                                      timestamp)).mkdir(parents=True,
                                                        exist_ok=True)
            file_path = os.path.join(self.output_path, "tmp_file", self.config_name, timestamp)
        else:
            file_path = os.path.join(self.output_path, "tmp_file", self.config_name)
        return file_path

    def get_train_model_out_dir(self, split_date=False):
        """
            获取训练模型文件保存路径 {output_path}/train_model/{config_name}/
        """
        pathlib.Path(os.path.join(self.output_path, "train_model", self.config_name)).mkdir(parents=True, exist_ok=True)
        if split_date:
            timestamp = "{0:%Y_%m_%d/}".format(datetime.now())
            pathlib.Path(os.path.join(self.output_path, "train_model", self.config_name, timestamp)).mkdir(parents=True,
                                                                                                           exist_ok=True)
            file_path = os.path.join(self.output_path, "train_model", self.config_name, timestamp)
        else:
            file_path = os.path.join(self.output_path, "train_model", self.config_name)
        return file_path

    def get_train_log_dir(self):
        """
            获取训练Tensorboard日志保存路径 {output_path}/train_log/{config_name}/timestamp
        """
        timestamp = "{0:%Y-%m-%d_%H-%M-%S/}".format(datetime.now())
        train_log_dir = os.path.join(self.output_path, "train_log", self.config_name, timestamp)
        return train_log_dir

    #
    # def get_train_data_dir(self):
    #     file_path = os.path.join(self.output_path, "train_model", self.config_name)
    #     return file_path



import numpy as np
import torch
import torch.nn as nn

from yqn_pytorch_framework.device import cpu_device


def init_cnn_weight(cnn_layer, seed=2233):
    """初始化cnn层权重
    Args:
        cnn_layer: weight.size() == [nb_filter, in_channels, [kernel_size]]
        seed: int
    """
    filter_nums = cnn_layer.weight.size(0)
    kernel_size = cnn_layer.weight.size()[2:]
    scope = np.sqrt(2. / (filter_nums * np.prod(kernel_size)))
    torch.manual_seed(seed)
    nn.init.normal_(cnn_layer.weight, -scope, scope)
    cnn_layer.bias.data.zero_()


def init_cnn(cnn_layer, seed=2233):
    """初始化cnn层权重
    Args:
        cnn_layer: weight.size() == [nb_filter, in_channels, [kernel_size]]
        seed: int
    """
    filter_nums = cnn_layer.weight.size(0)
    kernel_size = cnn_layer.weight.size()[2:]
    scope = np.sqrt(2. / (filter_nums * np.prod(kernel_size)))
    torch.manual_seed(seed)
    nn.init.xavier_normal_(cnn_layer.weight)
    cnn_layer.bias.data.uniform_(-scope, scope)


def init_lstm_weight(lstm, num_layer=1, seed=2233):
    """初始化lstm权重
    Args:
        lstm: torch.nn.LSTM
        num_layer: int, lstm层数
        seed: int
    """
    for i in range(num_layer):
        weight_h = getattr(lstm, 'weight_hh_l{0}'.format(i))
        scope = np.sqrt(6.0 / (weight_h.size(0) / 4. + weight_h.size(1)))
        torch.manual_seed(seed)
        nn.init.uniform_(getattr(lstm, 'weight_hh_l{0}'.format(i)), -scope, scope)

        weight_i = getattr(lstm, 'weight_ih_l{0}'.format(i))
        scope = np.sqrt(6.0 / (weight_i.size(0) / 4. + weight_i.size(1)))
        torch.manual_seed(seed)
        nn.init.uniform_(getattr(lstm, 'weight_ih_l{0}'.format(i)), -scope, scope)

    if lstm.bias:
        for i in range(num_layer):
            weight_h = getattr(lstm, 'bias_hh_l{0}'.format(i))
            weight_h.data.zero_()
            weight_h.data[lstm.hidden_size: 2 * lstm.hidden_size] = 1
            weight_i = getattr(lstm, 'bias_ih_l{0}'.format(i))
            weight_i.data.zero_()
            weight_i.data[lstm.hidden_size: 2 * lstm.hidden_size] = 1


def init_linear(input_linear, seed=2233):
    """初始化全连接层权重
    """
    torch.manual_seed(seed)
    scope = np.sqrt(6.0 / (input_linear.weight.size(0) + input_linear.weight.size(1)))
    nn.init.uniform_(input_linear.weight, -scope, scope)
    # nn.init.uniform(input_linear.bias, -scope, scope)
    if input_linear.bias is not None:
        input_linear.bias.data.zero_()


def init_linear_weight_bias(input_linear, seed=2233):
    """
    :param input_linear:
    :param seed:
    :return:
    """
    torch.manual_seed(seed)
    nn.init.xavier_uniform_(input_linear.weight)
    scope = np.sqrt(6.0 / (input_linear.weight.size(0) + 1))
    if input_linear.bias is not None:
        input_linear.bias.data.uniform_(-scope, scope)


def init_embedding(input_embedding, seed=5566):
    """初始化embedding层权重
    """
    torch.manual_seed(seed)
    scope = np.sqrt(8.0 / input_embedding.size(1))
    nn.init.uniform_(input_embedding, -scope, scope)


def init_embed(input_embedding, seed=5566):
    """初始化embedding层权重
    """
    torch.manual_seed(seed)
    nn.init.xavier_uniform_(input_embedding)


def prepare_pack_padded_sequence(inputs_words, seq_lengths, device=cpu_device, descending=True):
    """
    :param device:
    :param inputs_words:
    :param seq_lengths:
    :param descending:
    :return:
    """
    sorted_seq_lengths, indices = torch.sort(seq_lengths, dim=0, descending=descending)
    if device != cpu_device:
        sorted_seq_lengths, indices = sorted_seq_lengths.cuda(), indices.cuda()
    _, sorted_indices = torch.sort(indices.view(indices.size(0)), descending=False)
    sorted_inputs_words = inputs_words[sorted_indices]
    return sorted_inputs_words, sorted_seq_lengths.cpu().numpy().reshape(-1), sorted_indices



# -*- coding:utf-8 -*-
import re


def check_contain_chinese(check_str):
    for ch in check_str:
        if u'\u4e00' <= ch <= u'\u9fff':
            return True
    return False


def capitalize(name):
    return name.capitalize()


def clear_unified(s):
    s = re.sub("：", ":", re.sub("（", "(", re.sub("）", ")", re.sub("；", ";", re.sub("＆", "&", str(s))))))
    return s


def clear_number(s):
    return re.sub("[123456789]{1}", "0", s)


def is_str(s):
    return type(s) == str


def to_lowercase(s):
    return str.lower(s)


def to_uppercase(s):
    return str.upper(s)


def ends_with(s, suffix, ignore_case=False):
    """
    suffix: str, list, or tuple
    """
    if is_str(suffix):
        suffix = [suffix]
    suffix = list(suffix)
    if ignore_case:
        for idx, suf in enumerate(suffix):
            suffix[idx] = to_lowercase(suf)
        s = to_lowercase(s)
    suffix = tuple(suffix)
    return s.endswith(suffix)


def starts_with(s, prefix, ignore_case=False):
    """
    prefix: str, list, or tuple
    """
    if is_str(prefix):
        prefix = [prefix]
    prefix = list(prefix)
    if ignore_case:
        for idx, pre in enumerate(prefix):
            prefix[idx] = to_lowercase(pre)
        s = to_lowercase(s)
    prefix = tuple(prefix)
    return s.startswith(prefix)


def is_line_cross(line_1, line_2, ratio=0.9):
    if line_1[1] <= line_2[0] or line_1[0] >= line_2[1]:
        return False
    elif line_2[0] <= line_1[0] <= line_1[1] <= line_2[1] or line_1[0] <= line_2[0] <= line_2[1] <= line_1[1]:
        return True
    else:
        long_line = [min(line_1[0], line_2[0]), max(line_1[1], line_2[1])]
        short_line = [max(line_1[0], line_2[0]), min(line_1[1], line_2[1])]
        the_ratio = (short_line[1] - short_line[0]) / (long_line[1] - long_line[0])
        if the_ratio >= ratio:
            return True
        else:
            return False


def text_blocks_sort(data, ratio=0.5):
    """
    :param data: [{'v_5-13': [1570, 1212, 1682, 1254]}, {'v_5-14': [1810, 1104, 1928, 1141]}, ...]
    :param ratio
    :return:
    """
    texts_blocks_sorted = []

    line_values_sorted_array = []
    line_index = []
    values_already = []
    for a in range(len(data)):
        a_info = data[a]
        a_index = list(a_info.keys())[0]
        a_points = list(a_info.values())[0]

        line_in_values_array = []
        line_in_values_index = []
        if a_index not in values_already:
            line_in_values_array.append({str(a_points[0]): a_info})
            line_in_values_index.append(a_points[0])
            values_already.append(a_index)
            for b in range(len(data)):
                b_info = data[b]
                b_index = list(b_info.keys())[0]
                b_points = list(b_info.values())[0]

                if a != b and b_index not in values_already and is_line_cross([a_points[1], a_points[3]],
                                                                              [b_points[1], b_points[3]], ratio):
                    line_in_values_array.append({str(b_points[0]): b_info})
                    line_in_values_index.append(b_points[0])
                    values_already.append(b_index)

            line_sorted_index = sorted(line_in_values_index, reverse=False)
            line_in_values_sorted = []
            for tmp_index in line_sorted_index:
                for tmp_v in line_in_values_array:
                    tmp_v_index = list(tmp_v.keys())[0]
                    if tmp_v_index == str(tmp_index):
                        line_in_values_sorted.append(list(tmp_v.values())[0])
                        line_in_values_array.remove(tmp_v)
                    else:
                        continue
            line_values_sorted_array.append({str(a_points[1]): line_in_values_sorted})
            line_index.append(a_points[1])
        else:
            continue

    sorted_index = sorted(line_index)
    for t_index in sorted_index:
        for t_v in line_values_sorted_array:
            t_v_index = list(t_v.keys())[0]
            if t_v_index == str(t_index):
                texts_blocks_sorted.append(list(t_v.values())[0])
                line_values_sorted_array.remove(t_v)
            else:
                continue

    return texts_blocks_sorted


def is_contain_by_iou(text, cell, ratio=0.5):
    width1 = abs(text[2] - text[0])
    height1 = abs(text[1] - text[3])
    width2 = abs(cell[2] - cell[0])
    height2 = abs(cell[1] - cell[3])
    x_max = max(text[0], text[2], cell[0], cell[2])
    y_max = max(text[1], text[3], cell[1], cell[3])
    x_min = min(text[0], text[2], cell[0], cell[2])
    y_min = min(text[1], text[3], cell[1], cell[3])
    iou_width = x_min + width1 + width2 - x_max
    iou_height = y_min + height1 + height2 - y_max
    if iou_width <= 0 or iou_height <= 0:
        return False
    else:
        iou_area = iou_width * iou_height  # 交集的面积
        box1_area = width1 * height1
        iou_ratio_new = iou_area / box1_area
        if iou_ratio_new > ratio:
            return True
        else:
            return False



import pathlib
import os
import numpy as np
import cv2
import glob
def origin_text_coordiction_map(origin_text,scale):
    '''
    从pdf解析的在pdf上的坐标，映射到图片上的坐标
    :param origin_text:dict
    :param scale: float
    :return: dict
    '''
    maped_origin_text=origin_text.copy()
    text_infos=origin_text["text_infos"]
    maped_text_infos=[]
    for text_info in text_infos:
        maped_text_info={}
        maped_text_rectangle={}
        text_rectangle=text_info["text_rectangle"]
        text=text_info["text"]
        llx=text_rectangle["llx"]
        lly=text_rectangle["lly"]
        urx=text_rectangle["urx"]
        ury=text_rectangle["ury"]
        maped_llx=int(llx*scale)
        maped_lly=int(lly*scale)
        maped_urx=int(urx*scale)
        maped_ury=int(ury*scale)
        maped_text_rectangle["llx"]=maped_llx
        maped_text_rectangle["lly"]=maped_lly
        maped_text_rectangle["urx"]=maped_urx
        maped_text_rectangle["ury"]=maped_ury
        maped_text_info["text"]=text
        maped_text_info["text_rectangle"]=maped_text_rectangle
        maped_text_infos.append(maped_text_info)
    maped_origin_text["text_infos"]=maped_text_infos
    return maped_origin_text

def coordition_four_map_eight(text_rectangle):
    '''
    从4坐标映射到8坐标
    :param text_rectangle:
    :return: []
    '''
    llx=text_rectangle["llx"]
    lly=text_rectangle["lly"]
    urx=text_rectangle["urx"]
    ury=text_rectangle["ury"]
    x1,y1=llx,lly
    x2,y2=urx,lly
    x3,y3=urx,ury
    x4,y4=llx,ury
    return [x1,y1,x2,y2,x3,y3,x4,y4]
def generator_data_from_origin_text(origin_text,scale,file_name,page_num,save_dir="/data-hdd/LingYue/ocr_data/txt"):
    '''
    从pdf解析结果 生成ocr训练数据
    :param origin_text:
    :return:
    '''
    origin_text=origin_text_coordiction_map(origin_text,scale)
    text_infos=origin_text["text_infos"]
    write_file_path=os.path.join(save_dir,file_name+"_"+str(page_num)+".txt")
    if os.path.exists(write_file_path):
        os.remove(write_file_path)
    with open(write_file_path,mode="a+",encoding="utf-8") as f:
        for text_info in text_infos:
            text=text_info["text"]
            text_rectangle=text_info["text_rectangle"]
            text_rectangle=coordition_four_map_eight(text_rectangle)
            text_rectangle=np.array(text_rectangle).astype(str)
            in_file_text=",".join(text_rectangle)+","+text
            f.write(in_file_text)
            f.write("\n")
def draw_rect_to_img(origin_text_path,img_path,save_dir="/data-hdd/LingYue/ocr_data"):
    d=pathlib.Path(origin_text_path)
    file_name=d.stem
    bboxes=[]
    texts=[]
    with open(origin_text_path,encoding="utf-8") as f:
        lines=f.readlines()
        for line in lines:
            if len(line)>0:
                splited_line=line.split(",")
                if len(splited_line)>0:
                    box=list(map(int,splited_line[:8]))
                    bboxes.append(box)
                    text=str(splited_line[8])
                    texts.append(text)
    img=cv2.imread(img_path)
    real_write_file_path=os.path.join(save_dir,"img",file_name+".jpg")
    cv2.imwrite(real_write_file_path,img)
    for box in bboxes:
        box=np.reshape(box,(4,2))
        cv2.drawContours(img, [box],-1, (255, 0, 0), 2)
    bboxes_write_file_path=os.path.join(save_dir,"mask",file_name+"_result.jpg")
    print(file_name)
    cv2.imwrite(bboxes_write_file_path,img)


def analysis_bbox_to_img(ocr_text_dir):
    for path in glob.glob(os.path.join(ocr_text_dir,"*.txt"),recursive=True):
        d=pathlib.Path(path)
        file_name=d.stem
        splited_file_name=file_name.split("_")
        ship_company=splited_file_name[0]
        img_name=splited_file_name[1]
        page_num=splited_file_name[2]
        img_dir=os.path.join("/data-hdd/LingYue/booking_confirmation/base_class",ship_company,"doc2img")
        real_img_name=str(page_num)+"#"+str(img_name)+".jpg"
        img_path=os.path.join(img_dir,real_img_name)
        draw_rect_to_img(path,img_path)
        print("deal end ",path)
def from_file_find_bboxes_and_text(file_path):
    bboxes=[]
    texts=[]
    with open(file_path,encoding="utf-8") as file:
        lines=file.readlines()
        for line in lines:
            if len(line) > 0:
                splited_line = line.split(",")
                if len(splited_line) > 0:
                    box = list(map(int, splited_line[:8]))
                    bboxes.append(box)
                    text = str(splited_line[8]).strip("\n")
                    texts.append(text)
    return bboxes,texts

def generator_crnn_data_from_box_to_img(ocr_data_dir):
    save_lable_path=os.path.join(ocr_data_dir,"crnn_less_20_txt","gen.txt")
    save_img_dir=os.path.join(ocr_data_dir,"crnn_less_20_img")
    text_dir=os.path.join(ocr_data_dir,"txt")
    img_dir=os.path.join(ocr_data_dir,"img")
    with open(save_lable_path,"a+",encoding="utf-8") as f:
        for path in glob.glob(os.path.join(text_dir,"*.txt"),recursive=True):
            d=pathlib.Path(path)
            file_name=d.stem
            img_path=os.path.join(img_dir,file_name+".jpg")
            img=cv2.imread(img_path)
            bboxes,texts=from_file_find_bboxes_and_text(path)
            for i in range(len(bboxes)):
                box=bboxes[i]
                text=texts[i]

                img_id=i+1
                if len(text) <= 71:
                    try:
                        cropped_img, (topleft_x, topleft_y, bot_right_x, bot_right_y)=crop_4(img,box)
                        this_img_path=os.path.join(save_img_dir,file_name+"_croped_"+str(img_id)+".jpg")
                        cv2.imwrite(this_img_path,cropped_img)
                        f.write(file_name+"_croped_"+str(img_id)+".jpg★✿★"+text)
                        f.write("\n")
                    except Exception as e:
                        continue
            print("deal end "+path)




def crop_4(img,bbox):
    img_h,img_w=img.shape[:2]
    bbox = np.reshape(bbox,(4,2))
    topleft_x = np.min(bbox[:, 0])
    topleft_y = np.min(bbox[:, 1])
    bot_right_x = np.max(bbox[:, 0])
    bot_right_y = np.max(bbox[:, 1])
    # img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)#测试使用
    topleft_x=topleft_x-1 if (topleft_x-1)>0 else 0
    bot_right_x=bot_right_x+1 if (bot_right_x+1)<img_w else img_w
    topleft_y=topleft_y-3 if (topleft_y-3)>0 else 0
    bot_right_y=bot_right_y+3 if (bot_right_y+3)<img_h else img_h
    cropped_img = img[topleft_y:bot_right_y, topleft_x:bot_right_x]
    h, w, _ = cropped_img.shape
    # print("h,w",h,w)
    if h>32.0:
        scale=h/32.0
        new_w,new_h=int(w/scale),int(h/scale)
        # print(new_w,new_h)
        if new_w>720:
            cropped_img=cv2.resize(cropped_img,(0,0),fx=720/w,fy=new_h/h,interpolation=cv2.INTER_AREA)
        else:
            cropped_img = cv2.resize(cropped_img, (0, 0), fx=new_w / w, fy=new_h / h, interpolation=cv2.INTER_AREA)
            cropped_img = cv2.copyMakeBorder(cropped_img, 0, 0, 0, (720 - new_w), cv2.BORDER_CONSTANT,
                                             value=(255, 255, 255))
    else:
        top_dis=int((32-h)/2)
        bot_dis=(32-h)-top_dis
        cropped_img = cv2.copyMakeBorder(cropped_img, top_dis, bot_dis, 0, 0, cv2.BORDER_CONSTANT,
                                         value=(255, 255, 255))
        if w>720:
            cropped_img=cv2.resize(cropped_img,(0,0),fx=720/w,fy=1,interpolation=cv2.INTER_AREA)
        else:
            cropped_img = cv2.copyMakeBorder(cropped_img, 0, 0, 0, (720 - w), cv2.BORDER_CONSTANT,
                                             value=(255, 255, 255))

    return cropped_img, (topleft_x, topleft_y, bot_right_x, bot_right_y)
def remove_no_parse_file(save_dir,no_parse_list):
    img_dir=os.path.join(save_dir,"crnn_img_resize")
    save_path=os.path.join(save_dir,"crnn_txt_resize","gen_cleared.txt")
    txt_path=os.path.join(save_dir,"crnn_txt_resize","gen.txt")
    for no_parse_info in no_parse_list:
        img_name=no_parse_info.split("★✿★")[0]
        img_path=os.path.join(img_dir,img_name)
        if os.path.exists(img_path):
            os.remove(img_path)
            print("删除图片:"+img_path)
    cleared_txt=[]
    with open(txt_path,encoding="utf-8") as f:
        lines=f.readlines()
        for no_parse_info in no_parse_list:
            if no_parse_info in lines:
                lines.remove(no_parse_info)
                print("删除信息："+no_parse_info)
        with open(save_path,"a+",encoding="utf-8") as file:
            for line in lines:
                line=line.strip("\n")
                file.write(line)
                file.write("\n")



if __name__=="__main__":
    # remove_no_parse_file("/data-hdd/LingYue/ocr_data",['emc_0e8cd06ab65248f2b4522cfc5b2a536e_1_croped_51.jpg★✿★/40© HI-CUBE\n', 'maersk_2086459_1_croped_28.jpg★✿★❤ 如有任何疑问，请联系我司客服邮箱 cn.east.export@maersk.com\n', 'emc_1bc1b66baeb241058679d2770a93ddd9_2_croped_9.jpg★✿★CHINA SAILING INT©L SHIPPING AGENCY LTD\n', 'cma_0c1b2ddddad74e27a226705e3d30f57d_1_croped_66.jpg★✿★• Part or all of the cargo actually stuffed inside the container is dangerous cargo\n', 'cma_0c1b2ddddad74e27a226705e3d30f57d_1_croped_67.jpg★✿★• The name\n', 'cma_0c1b2ddddad74e27a226705e3d30f57d_1_croped_68.jpg★✿★• The cargo is declared as general cargo at the time of booking but is requested to be amended to dangerous cargo after our release of the empty container.\n', 'cma_0c1b2ddddad74e27a226705e3d30f57d_1_croped_69.jpg★✿★• The cargo is discovered or determined by any domestic or foreign authority (including but not limited to customs\n', 'emc_0b9c4993fc3140bfb5aed728f6c5f0e7_2_croped_9.jpg★✿★CHINA SAILING INT©L SHIPPING AGENCY LTD\n', 'maersk_2146271_1_croped_28.jpg★✿★❤ 如有任何疑问，请联系我司客服邮箱 cn.east.export@maersk.com\n', 'emc_2c2e212c89c84e4282950917b140fd4f_2_croped_9.jpg★✿★CHINA SAILING INT©L SHIPPING AGENCY LTD\n', 'emc_1fee5be6a2794ee28de514f115d88103_1_croped_49.jpg★✿★/40© HI-CUBE\n', 'emc_1c86c7d6fb2740b6b1763f041d008a96_2_croped_8.jpg★✿★CHINA SAILING INT©L SHIPPING AGENCY LTD\n', 'emc_1b0c4b9d70ea46cea18e6f0379947cbb_2_croped_14.jpg★✿★相关费率请至ªwww.master-agency.com.cn---->费率---->提单各项费用º查询。\n', 'emc_1b0c4b9d70ea46cea18e6f0379947cbb_2_croped_14.jpg★✿★相关费率请至ªwww.master-agency.com.cn---->费率---->提单各项费用º查询。\n', 'cma_5b75b8a96ba542d98844ea282bfc51bb_2_croped_57.jpg★✿★• Part or all of the cargo actually stuffed inside the container is dangerous cargo\n', 'cma_5b75b8a96ba542d98844ea282bfc51bb_2_croped_59.jpg★✿★• The name\n', 'cma_5b75b8a96ba542d98844ea282bfc51bb_2_croped_62.jpg★✿★• The cargo is declared as general cargo at the time of booking but is requested to be amended to dangerous cargo after our release of the empty container.\n', 'cma_5b75b8a96ba542d98844ea282bfc51bb_2_croped_64.jpg★✿★• The cargo is discovered or determined by any domestic or foreign authority (including but not limited to customs\n', 'maersk_2075207_1_croped_56.jpg★✿★运输方式\xa0船名\n', 'emc_1b0c4b9d70ea46cea18e6f0379947cbb_1_croped_52.jpg★✿★/40© HI-CUBE\n', 'cma_7e17ae524af94f71836904fc28878025_2_croped_62.jpg★✿★• Part or all of the cargo actually stuffed inside the container is dangerous cargo\n', 'cma_7e17ae524af94f71836904fc28878025_2_croped_64.jpg★✿★• The name\n', 'cma_7e17ae524af94f71836904fc28878025_2_croped_67.jpg★✿★• The cargo is declared as general cargo at the time of booking but is requested to be amended to dangerous cargo after our release of the empty container.\n', 'cma_7e17ae524af94f71836904fc28878025_2_croped_69.jpg★✿★• The cargo is discovered or determined by any domestic or foreign authority (including but not limited to customs\n', 'maersk_2279463_1_croped_28.jpg★✿★❤ 如有任何疑问，请联系我司客服邮箱 cn.east.export@maersk.com\n', 'maersk_2026074_1_croped_28.jpg★✿★❤ 如有任何疑问，请联系我司客服邮箱 cn.east.export@maersk.com\n', 'maersk_2026074_1_croped_82.jpg★✿★❤重要提示：\n', 'cma_23218e1256ca485bb4fbc6aacf4adc65_2_croped_62.jpg★✿★• Part or all of the cargo actually stuffed inside the container is dangerous cargo\n', 'cma_23218e1256ca485bb4fbc6aacf4adc65_2_croped_64.jpg★✿★• The name\n', 'cma_23218e1256ca485bb4fbc6aacf4adc65_2_croped_67.jpg★✿★• The cargo is declared as general cargo at the time of booking but is requested to be amended to dangerous cargo after our release of the empty container.\n', 'cma_23218e1256ca485bb4fbc6aacf4adc65_2_croped_69.jpg★✿★• The cargo is discovered or determined by any domestic or foreign authority (including but not limited to customs\n', 'cma_3472b11cb89c40e9bffa226413c42037_2_croped_62.jpg★✿★• Part or all of the cargo actually stuffed inside the container is dangerous cargo\n', 'cma_3472b11cb89c40e9bffa226413c42037_2_croped_64.jpg★✿★• The name\n', 'cma_3472b11cb89c40e9bffa226413c42037_2_croped_67.jpg★✿★• The cargo is declared as general cargo at the time of booking but is requested to be amended to dangerous cargo after our release of the empty container.\n', 'cma_3472b11cb89c40e9bffa226413c42037_2_croped_69.jpg★✿★• The cargo is discovered or determined by any domestic or foreign authority (including but not limited to customs\n', 'emc_01d27221106f44548b1642e7359d449a_1_croped_42.jpg★✿★/40© HI-CUBE\n', 'maersk_2305321_1_croped_28.jpg★✿★❤ 如有任何疑问，请联系我司客服邮箱 cn.east.export@maersk.com\n', 'maersk_2086459_2_croped_2.jpg★✿★❤重要提示：\n', 'emc_0a0201f857884c50adf375fb7a52f2db_1_croped_46.jpg★✿★/40© HI-CUBE\n', 'cma_761a46d5b640498f8d15843349a81cb9_2_croped_62.jpg★✿★• Part or all of the cargo actually stuffed inside the container is dangerous cargo\n', 'cma_761a46d5b640498f8d15843349a81cb9_2_croped_64.jpg★✿★• The name\n', 'cma_761a46d5b640498f8d15843349a81cb9_2_croped_67.jpg★✿★• The cargo is declared as general cargo at the time of booking but is requested to be amended to dangerous cargo after our release of the empty container.\n', 'cma_761a46d5b640498f8d15843349a81cb9_2_croped_69.jpg★✿★• The cargo is discovered or determined by any domestic or foreign authority (including but not limited to customs\n', 'cma_3199176333f4419d9116bd5fe108e3fe_1_croped_70.jpg★✿★• Part or all of the cargo actually stuffed inside the container is dangerous cargo\n', 'cma_3199176333f4419d9116bd5fe108e3fe_1_croped_71.jpg★✿★• The name\n', 'cma_3199176333f4419d9116bd5fe108e3fe_1_croped_72.jpg★✿★• The cargo is declared as general cargo at the time of booking but is requested to be amended to dangerous cargo after our release of the empty container.\n', 'cma_3199176333f4419d9116bd5fe108e3fe_1_croped_73.jpg★✿★• The cargo is discovered or determined by any domestic or foreign authority (including but not limited to customs\n', 'emc_0d5161f592ec48e2b87f171e702e9005_1_croped_68.jpg★✿★/40© STANDARD DRY\n', 'emc_1bc1b66baeb241058679d2770a93ddd9_1_croped_46.jpg★✿★/20© STANDARD DRY\n', 'maersk_2076946_1_croped_28.jpg★✿★❤ 如有任何疑问，请联系我司客服邮箱 cn.east.export@maersk.com\n', 'maersk_2076946_1_croped_81.jpg★✿★❤重要提示：\n', 'emc_2bd8aab2ca8747408a6fe393d0696164_1_croped_34.jpg★✿★:LEXUS PART©S ACCESSORIES CO.LTD\n', 'emc_2bd8aab2ca8747408a6fe393d0696164_1_croped_48.jpg★✿★/40© HI-CUBE\n', 'maersk_2305326_1_croped_27.jpg★✿★❤ 如有任何疑问，请联系我司客服邮箱 cn.east.export@maersk.com\n', 'maersk_2305326_1_croped_80.jpg★✿★❤重要提示：\n', 'hpl_0fc573ec9d7b4f159a7584bdf31fb5db_1_croped_227.jpg★✿★Nicolás\n', 'emc_2cf311e18a124fb0b48cd19fbfb1bfbe_1_croped_34.jpg★✿★:LEXUS PART©S ACCESSORIES CO.LTD\n', 'emc_2cf311e18a124fb0b48cd19fbfb1bfbe_1_croped_48.jpg★✿★/40© HI-CUBE\n', 'emc_0dc23387b6204ca28613fcc13462e35b_1_croped_48.jpg★✿★/40© HI-CUBE\n', 'emc_0fdaa0b32c514630a07c41ceea1fbad5_1_croped_46.jpg★✿★/40© HI-CUBE\n', 'maersk_2004842_1_croped_79.jpg★✿★❤重要提示：\n', 'maersk_2056172_1_croped_28.jpg★✿★❤ 如有任何疑问，请联系我司客服邮箱 cn.east.export@maersk.com\n', 'maersk_2056172_1_croped_82.jpg★✿★❤重要提示：\n', 'cma_5b4ade6e4d534734aaaec9ed9b127598_1_croped_65.jpg★✿★• Part or all of the cargo actually stuffed inside the container is dangerous cargo\n', 'cma_5b4ade6e4d534734aaaec9ed9b127598_1_croped_66.jpg★✿★• The name\n', 'cma_5b4ade6e4d534734aaaec9ed9b127598_1_croped_67.jpg★✿★• The cargo is declared as general cargo at the time of booking but is requested to be amended to dangerous cargo after our release of the empty container.\n', 'cma_5b4ade6e4d534734aaaec9ed9b127598_1_croped_68.jpg★✿★• The cargo is discovered or determined by any domestic or foreign authority (including but not limited to customs\n', 'hpl_1c1369e9d96a40728ff9250b656ede45_1_croped_45.jpg★✿★✖\n', 'emc_1aa947b72aff47c4a1cee350b0bf0bbf_1_croped_46.jpg★✿★/40© HI-CUBE\n', 'hpl_1fa5c07ee92b43538c622d55b435a9b9_1_croped_51.jpg★✿★✖\n', 'emc_1c86c7d6fb2740b6b1763f041d008a96_1_croped_46.jpg★✿★/20© STANDARD DRY\n', 'maersk_2049122_1_croped_31.jpg★✿★❤ 如有任何疑问，请联系我司客服邮箱 cn.east.export@maersk.com\n', 'emc_0dc23387b6204ca28613fcc13462e35b_2_croped_9.jpg★✿★CHINA SAILING INT©L SHIPPING AGENCY LTD\n', 'maersk_2146271_2_croped_2.jpg★✿★❤重要提示：\n', 'maersk_2023161_2_croped_9.jpg★✿★❤重要提示：\n', 'maersk_2070396_1_croped_28.jpg★✿★❤ 如有任何疑问，请联系我司客服邮箱 cn.east.export@maersk.com\n', 'maersk_2070396_1_croped_81.jpg★✿★❤重要提示：\n', 'mcc_7174bb7657c04a6aacbbac465b00b193_1_croped_32.jpg★✿★\xa0\xa0\xa0\n', 'mcc_7174bb7657c04a6aacbbac465b00b193_1_croped_32.jpg★✿★\xa0\xa0\xa0\n', 'mcc_7174bb7657c04a6aacbbac465b00b193_1_croped_32.jpg★✿★\xa0\xa0\xa0\n', 'emc_2bd8aab2ca8747408a6fe393d0696164_2_croped_9.jpg★✿★CHINA SAILING INT©L SHIPPING AGENCY LTD\n', 'emc_2cf311e18a124fb0b48cd19fbfb1bfbe_2_croped_9.jpg★✿★CHINA SAILING INT©L SHIPPING AGENCY LTD\n', 'cma_24dece82863f4bb1b6ee747f72e7ed24_2_croped_62.jpg★✿★• Part or all of the cargo actually stuffed inside the container is dangerous cargo\n', 'cma_24dece82863f4bb1b6ee747f72e7ed24_2_croped_64.jpg★✿★• The name\n', 'cma_24dece82863f4bb1b6ee747f72e7ed24_2_croped_67.jpg★✿★• The cargo is declared as general cargo at the time of booking but is requested to be amended to dangerous cargo after our release of the empty container.\n', 'cma_24dece82863f4bb1b6ee747f72e7ed24_2_croped_69.jpg★✿★• The cargo is discovered or determined by any domestic or foreign authority (including but not limited to customs\n', 'emc_0ad46a3b2cd74e2b943f26624aa17936_1_croped_46.jpg★✿★/40© HI-CUBE\n', 'cma_26a1fcf190444fd4baf37778da8175c8_2_croped_62.jpg★✿★• Part or all of the cargo actually stuffed inside the container is dangerous cargo\n', 'cma_26a1fcf190444fd4baf37778da8175c8_2_croped_64.jpg★✿★• The name\n', 'cma_26a1fcf190444fd4baf37778da8175c8_2_croped_67.jpg★✿★• The cargo is declared as general cargo at the time of booking but is requested to be amended to dangerous cargo after our release of the empty container.\n', 'cma_26a1fcf190444fd4baf37778da8175c8_2_croped_69.jpg★✿★• The cargo is discovered or determined by any domestic or foreign authority (including but not limited to customs\n', 'emc_0fdaa0b32c514630a07c41ceea1fbad5_2_croped_8.jpg★✿★CHINA SAILING INT©L SHIPPING AGENCY LTD\n', 'cma_0e22cf28dc5447608fc29d419335edff_2_croped_62.jpg★✿★• Part or all of the cargo actually stuffed inside the container is dangerous cargo\n', 'cma_0e22cf28dc5447608fc29d419335edff_2_croped_64.jpg★✿★• The name\n', 'cma_0e22cf28dc5447608fc29d419335edff_2_croped_67.jpg★✿★• The cargo is declared as general cargo at the time of booking but is requested to be amended to dangerous cargo after our release of the empty container.\n', 'cma_0e22cf28dc5447608fc29d419335edff_2_croped_69.jpg★✿★• The cargo is discovered or determined by any domestic or foreign authority (including but not limited to customs\n', 'hpl_0fc573ec9d7b4f159a7584bdf31fb5db_2_croped_426.jpg★✿★Nicolás\n', 'maersk_2048145_1_croped_31.jpg★✿★❤ 如有任何疑问，请联系我司客服邮箱 cn.export@safmarine.com\n', 'maersk_2048145_1_croped_63.jpg★✿★运输方式\xa0船名\n', 'emc_1db69441bcd44dfc8f7f4005d8114f3f_1_croped_51.jpg★✿★/40© HI-CUBE\n', 'cma_933ca7cd667741ad837040427e0744d9_2_croped_62.jpg★✿★• Part or all of the cargo actually stuffed inside the container is dangerous cargo\n', 'cma_933ca7cd667741ad837040427e0744d9_2_croped_64.jpg★✿★• The name\n', 'cma_933ca7cd667741ad837040427e0744d9_2_croped_67.jpg★✿★• The cargo is declared as general cargo at the time of booking but is requested to be amended to dangerous cargo after our release of the empty container.\n', 'cma_933ca7cd667741ad837040427e0744d9_2_croped_69.jpg★✿★• The cargo is discovered or determined by any domestic or foreign authority (including but not limited to customs\n', 'emc_2d1f1b30244b407190bf8d2726983e69_1_croped_48.jpg★✿★/40© STANDARD DRY\n', 'cma_36ebff9647df4b9b8024735e56e43647_2_croped_62.jpg★✿★• Part or all of the cargo actually stuffed inside the container is dangerous cargo\n', 'cma_36ebff9647df4b9b8024735e56e43647_2_croped_64.jpg★✿★• The name\n', 'cma_36ebff9647df4b9b8024735e56e43647_2_croped_67.jpg★✿★• The cargo is declared as general cargo at the time of booking but is requested to be amended to dangerous cargo after our release of the empty container.\n', 'cma_36ebff9647df4b9b8024735e56e43647_2_croped_69.jpg★✿★• The cargo is discovered or determined by any domestic or foreign authority (including but not limited to customs\n', 'hpl_1cba18c4ba0e40d282dce9a15a38d551_1_croped_45.jpg★✿★✖\n', 'emc_1dd096d2429b4f289e6a7ef11b81235a_1_croped_34.jpg★✿★:LEXUS PART©S ACCESSORIES CO.LTD\n', 'emc_1dd096d2429b4f289e6a7ef11b81235a_1_croped_50.jpg★✿★/40© HI-CUBE\n', 'cma_8c3e93ea9b7a480181a2a16906f05315_2_croped_57.jpg★✿★• Part or all of the cargo actually stuffed inside the container is dangerous cargo\n', 'cma_8c3e93ea9b7a480181a2a16906f05315_2_croped_59.jpg★✿★• The name\n', 'cma_8c3e93ea9b7a480181a2a16906f05315_2_croped_62.jpg★✿★• The cargo is declared as general cargo at the time of booking but is requested to be amended to dangerous cargo after our release of the empty container.\n', 'cma_8c3e93ea9b7a480181a2a16906f05315_2_croped_64.jpg★✿★• The cargo is discovered or determined by any domestic or foreign authority (including but not limited to customs\n', 'cma_64d01129d0194a6f9b8c5fdbe55cb2de_2_croped_53.jpg★✿★• Part or all of the cargo actually stuffed inside the container is dangerous cargo\n', 'cma_64d01129d0194a6f9b8c5fdbe55cb2de_2_croped_55.jpg★✿★• The name\n', 'cma_64d01129d0194a6f9b8c5fdbe55cb2de_2_croped_58.jpg★✿★• The cargo is declared as general cargo at the time of booking but is requested to be amended to dangerous cargo after our release of the empty container.\n', 'cma_64d01129d0194a6f9b8c5fdbe55cb2de_2_croped_60.jpg★✿★• The cargo is discovered or determined by any domestic or foreign authority (including but not limited to customs\n', 'hpl_0fc573ec9d7b4f159a7584bdf31fb5db_3_croped_208.jpg★✿★Nicolás\n', 'cma_6a44381bace9402e945a132e121e6bcd_2_croped_53.jpg★✿★• Part or all of the cargo actually stuffed inside the container is dangerous cargo\n', 'cma_6a44381bace9402e945a132e121e6bcd_2_croped_55.jpg★✿★• The name\n', 'cma_6a44381bace9402e945a132e121e6bcd_2_croped_58.jpg★✿★• The cargo is declared as general cargo at the time of booking but is requested to be amended to dangerous cargo after our release of the empty container.\n', 'cma_6a44381bace9402e945a132e121e6bcd_2_croped_60.jpg★✿★• The cargo is discovered or determined by any domestic or foreign authority (including but not limited to customs\n', 'emc_1e536c150b844858900c5afc7164fc06_1_croped_42.jpg★✿★:AND COUNT ARRASADOR 75.7 SG(GLYPHOSATE 75.7 SG)CUSTOMER©S REF.STK01796\n', 'emc_1e536c150b844858900c5afc7164fc06_1_croped_52.jpg★✿★/40© HI-CUBE\n', 'emc_1fee5be6a2794ee28de514f115d88103_2_croped_9.jpg★✿★CHINA SAILING INT©L SHIPPING AGENCY LTD\n', 'emc_0dcf14eb93534f78acaccb2c514389a7_1_croped_46.jpg★✿★/20© STANDARD DRY\n', 'maersk_2069643_1_croped_62.jpg★✿★运输方式\xa0船名\n', 'maersk_2049122_2_croped_2.jpg★✿★❤重要提示：\n', 'maersk_2066349_1_croped_28.jpg★✿★❤ 如有任何疑问，请联系我司客服邮箱 cn.east.export@maersk.com\n', 'maersk_2066349_1_croped_82.jpg★✿★❤重要提示：\n', 'cma_2f31721f7e384a9da949e011f82b96c7_2_croped_59.jpg★✿★• Part or all of the cargo actually stuffed inside the container is dangerous cargo\n', 'cma_2f31721f7e384a9da949e011f82b96c7_2_croped_61.jpg★✿★• The name\n', 'cma_2f31721f7e384a9da949e011f82b96c7_2_croped_64.jpg★✿★• The cargo is declared as general cargo at the time of booking but is requested to be amended to dangerous cargo after our release of the empty container.\n', 'cma_2f31721f7e384a9da949e011f82b96c7_2_croped_66.jpg★✿★• The cargo is discovered or determined by any domestic or foreign authority (including but not limited to customs\n', 'emc_0cb2f5768bb447f39c6f1fd057794ecb_1_croped_47.jpg★✿★/40© HI-CUBE\n', 'maersk_2006751_1_croped_32.jpg★✿★❤ 如有任何疑问，请联系我司客服邮箱 cn.east.export@maersk.com\n', 'maersk_2006751_1_croped_92.jpg★✿★❤重要提示：\n', 'maersk_2283907_1_croped_30.jpg★✿★❤ 如有任何疑问，请联系我司客服邮箱 cn.east.export@maersk.com\n', 'emc_1c69f888269f4b2bb07e7b5328b5b7c9_1_croped_37.jpg★✿★:SHIPPER©S LOAD\n', 'emc_1c69f888269f4b2bb07e7b5328b5b7c9_1_croped_46.jpg★✿★/40© HI-CUBE\n', 'emc_01d27221106f44548b1642e7359d449a_2_croped_8.jpg★✿★CHINA SAILING INT©L SHIPPING AGENCY LTD\n', 'emc_1b257d0da7654660913f9548e33fa742_1_croped_42.jpg★✿★:1X40©HC(FCL) CY-CY S.T.C.115 CTNS OF SKD-108RAINBOW251 CTNS OF SKD-05 \n', 'emc_1b257d0da7654660913f9548e33fa742_1_croped_56.jpg★✿★/40© HI-CUBE\n', 'emc_2cd541f517cb44df9da0a852a13f8fa6_1_croped_21.jpg★✿★(8) Point and Country of Origin (for the Merchant＇\n', 'emc_2cd541f517cb44df9da0a852a13f8fa6_1_croped_48.jpg★✿★(M³)\n', 'emc_2cd541f517cb44df9da0a852a13f8fa6_1_croped_60.jpg★✿★GLDU5746415/20©/EMCWCX6728/14 PALLETS\n', 'emc_2cd541f517cb44df9da0a852a13f8fa6_1_croped_61.jpg★✿★1 X 20©\n', 'emc_2cd541f517cb44df9da0a852a13f8fa6_1_croped_65.jpg★✿★SHIPPER©S LOAD & COUNT\n', 'emc_2c2e212c89c84e4282950917b140fd4f_1_croped_34.jpg★✿★:LEXUS PART©S ACCESSORIES CO.LTD\n', 'emc_2c2e212c89c84e4282950917b140fd4f_1_croped_48.jpg★✿★/40© HI-CUBE\n', 'emc_0b9c4993fc3140bfb5aed728f6c5f0e7_1_croped_46.jpg★✿★/40© HI-CUBE\n', 'cma_0c9667e2e899465590d4921ba7688e7b_1_croped_64.jpg★✿★• Part or all of the cargo actually stuffed inside the container is dangerous cargo\n', 'cma_0c9667e2e899465590d4921ba7688e7b_1_croped_65.jpg★✿★• The name\n', 'cma_0c9667e2e899465590d4921ba7688e7b_1_croped_66.jpg★✿★• The cargo is declared as general cargo at the time of booking but is requested to be amended to dangerous cargo after our release of the empty container.\n', 'cma_0c9667e2e899465590d4921ba7688e7b_1_croped_67.jpg★✿★• The cargo is discovered or determined by any domestic or foreign authority (including but not limited to customs\n', 'emc_0fec96bc01104e19811c2ef5d7b6039a_1_croped_46.jpg★✿★/40© HI-CUBE\n', 'emc_0c0cadfeadca40e9aadd8a19ce1230c5_2_croped_9.jpg★✿★CHINA SAILING INT©L SHIPPING AGENCY LTD\n', 'emc_0c0cadfeadca40e9aadd8a19ce1230c5_1_croped_34.jpg★✿★:LEXUS PART©S ACCESSORIES CO.LTD\n', 'emc_0c0cadfeadca40e9aadd8a19ce1230c5_1_croped_48.jpg★✿★/40© HI-CUBE\n'])
    generator_crnn_data_from_box_to_img("/data-hdd/LingYue/ocr_data")
    # analysis_bbox_to_img("/data-hdd/LingYue/ocr_data/txt")
#     # origin_text={'file_id': 4490793, 'file_name': 'daca63b122a6460697c36d3bd170aafc.jpg', 'file_type': 'PDF_GROUP', 'file_url': 'http://sruserfiletest.oss-cn-hangzhou.aliyuncs.com/oms/file_convert/daca63b122a6460697c36d3bd170aafc.jpg?Expires=1586924447&OSSAccessKeyId=LTAI4FufopNQTpwzS98ThU38&Signature=XLpF4Scx4MQAbGGeLUzZTtw6c4c%3D', 'page_index': 1, 'page_width': 595.0, 'page_height': 842.0, 'image_info': {'page_index': 1, 'image_file_id': 4490793, 'height': 3508, 'width': 2479, 'image_path': 'http://sruserfiletest.oss-cn-hangzhou.aliyuncs.com/oms/file_convert/daca63b122a6460697c36d3bd170aafc.jpg?Expires=1586924447&OSSAccessKeyId=LTAI4FufopNQTpwzS98ThU38&Signature=XLpF4Scx4MQAbGGeLUzZTtw6c4c%3D'}, 'text_infos': [{'text': 'BOOKING AMENDMENT', 'text_rectangle': {'llx': 344.82, 'lly': 30.0339, 'urx': 534.036, 'ury': 47.6339, 'height': 17.6, 'width': 189.216}}, {'text': '订舱号:', 'text_rectangle': {'llx': 30.278, 'lly': 77.23805, 'urx': 56.502, 'ury': 86.03805, 'height': 8.799995, 'width': 26.223999}}, {'text': '打印时间：', 'text_rectangle': {'llx': 402.572, 'lly': 76.8018, 'urx': 442.572, 'ury': 85.6018, 'height': 8.799995, 'width': 40.0}}, {'text': '2019-10-21 08:46 UTC', 'text_rectangle': {'llx': 453.603, 'lly': 77.3873, 'urx': 534.243, 'ury': 86.1873, 'height': 8.800003, 'width': 80.639984}}, {'text': '587093467', 'text_rectangle': {'llx': 112.493, 'lly': 75.2284, 'urx': 181.289, 'ury': 88.4284, 'height': 13.199997, 'width': 68.796005}}, {'text': '订舱人:', 'text_rectangle': {'llx': 28.3502, 'lly': 94.436, 'urx': 54.5742, 'ury': 103.236, 'height': 8.800003, 'width': 26.223999}}, {'text': 'YUEHAI SUPPLY CHAIN SERVICE CO LTD', 'text_rectangle': {'llx': 108.439, 'lly': 94.397, 'urx': 253.223, 'ury': 103.197, 'height': 8.799995, 'width': 144.784}}, {'text': '交接方式:', 'text_rectangle': {'llx': 311.852, 'lly': 94.436, 'urx': 346.076, 'ury': 103.236, 'height': 8.800003, 'width': 34.224}}, {'text': 'CY/CY', 'text_rectangle': {'llx': 384.003, 'lly': 94.397, 'urx': 405.891, 'ury': 103.197, 'height': 8.799995, 'width': 21.888}}, {'text': '联系人:', 'text_rectangle': {'llx': 28.3502, 'lly': 105.776, 'urx': 54.5742, 'ury': 114.576, 'height': 8.799995, 'width': 26.223999}}, {'text': 'BESS LI', 'text_rectangle': {'llx': 108.439, 'lly': 105.256, 'urx': 135.935, 'ury': 114.056, 'height': 8.800003, 'width': 27.495995}}, {'text': '收货地:', 'text_rectangle': {'llx': 311.852, 'lly': 105.776, 'urx': 338.076, 'ury': 114.576, 'height': 8.799995, 'width': 26.223999}}, {'text': 'From:', 'text_rectangle': {'llx': 311.852, 'lly': 105.776, 'urx': 332.74, 'ury': 114.576, 'height': 8.799995, 'width': 20.888}}, {'text': 'Shanghai,Shanghai,China', 'text_rectangle': {'llx': 384.003, 'lly': 105.737, 'urx': 473.627, 'ury': 114.537, 'height': 8.800003, 'width': 89.62402}}, {'text': '订舱人参考号:', 'text_rectangle': {'llx': 28.2651, 'lly': 117.201, 'urx': 78.4891, 'ury': 126.001, 'height': 8.800003, 'width': 50.224}}, {'text': '交货地:', 'text_rectangle': {'llx': 311.852, 'lly': 117.116, 'urx': 338.076, 'ury': 125.916, 'height': 8.800003, 'width': 26.223999}}, {'text': 'To:', 'text_rectangle': {'llx': 311.852, 'lly': 117.116, 'urx': 323.412, 'ury': 125.916, 'height': 8.800003, 'width': 11.559998}}, {'text': 'Sydney,New South Wales,Australia', 'text_rectangle': {'llx': 384.003, 'lly': 117.078, 'urx': 505.427, 'ury': 125.878, 'height': 8.799995, 'width': 121.42401}}, {'text': '合约号:', 'text_rectangle': {'llx': 28.3502, 'lly': 128.456, 'urx': 54.5742, 'ury': 137.256, 'height': 8.800003, 'width': 26.223999}}, {'text': '54963584', 'text_rectangle': {'llx': 108.439, 'lly': 128.418, 'urx': 143.383, 'ury': 137.218, 'height': 8.800003, 'width': 34.943993}}, {'text': 'Customer Commodity:', 'text_rectangle': {'llx': 311.852, 'lly': 128.456, 'urx': 391.42, 'ury': 137.256, 'height': 8.800003, 'width': 79.56802}}, {'text': '54963584', 'text_rectangle': {'llx': 394.351, 'lly': 128.418, 'urx': 429.295, 'ury': 137.218, 'height': 8.800003, 'width': 34.944}}, {'text': '合约客户:', 'text_rectangle': {'llx': 28.3502, 'lly': 139.796, 'urx': 62.5742, 'ury': 148.596, 'height': 8.799988, 'width': 34.224}}, {'text': 'YUEHAI SUPPLY CHAIN SERVICE CO LTD', 'text_rectangle': {'llx': 108.439, 'lly': 139.7863, 'urx': 253.223, 'ury': 148.5863, 'height': 8.800003, 'width': 144.784}}, {'text': '受理订舱分公司:', 'text_rectangle': {'llx': 311.852, 'lly': 139.796, 'urx': 370.076, 'ury': 148.596, 'height': 8.799988, 'width': 58.224}}, {'text': 'Business Unit:', 'text_rectangle': {'llx': 311.852, 'lly': 139.796, 'urx': 362.98, 'ury': 148.596, 'height': 8.799988, 'width': 51.12802}}, {'text': 'Maersk China Shipping (Qingdao)', 'text_rectangle': {'llx': 384.003, 'lly': 139.758, 'urx': 502.091, 'ury': 148.558, 'height': 8.800003, 'width': 118.08801}}, {'text': '参考号:', 'text_rectangle': {'llx': 28.3502, 'lly': 151.136, 'urx': 54.5742, 'ury': 159.936, 'height': 8.800003, 'width': 26.223999}}, {'text': 'Commodity Description:', 'text_rectangle': {'llx': 311.795, 'lly': 150.966, 'urx': 396.699, 'ury': 159.766, 'height': 8.800003, 'width': 84.90399}}, {'text': 'Chemical products, nos', 'text_rectangle': {'llx': 397.271, 'lly': 151.2113, 'urx': 479.255, 'ury': 160.0113, 'height': 8.800003, 'width': 81.98401}}, {'text': 'We request you to review the specific parameters, viz. Service Contract, Price Owner, Named account customer and', 'text_rectangle': {'llx': 29.1723, 'lly': 163.728, 'urx': 550.8923, 'ury': 172.528, 'height': 8.800003, 'width': 521.72}}, {'text': 'Commodity description. In case there are any changes required to these parameters, please send us a request before', 'text_rectangle': {'llx': 29.1723, 'lly': 173.328, 'urx': 557.9403, 'ury': 182.128, 'height': 8.800003, 'width': 528.768}}, {'text': 'any containers(s) are picked up.', 'text_rectangle': {'llx': 29.1723, 'lly': 182.928, 'urx': 175.2603, 'ury': 191.728, 'height': 8.800003, 'width': 146.088}}, {'text': '"***************** [截操作时间参考] *****************', 'text_rectangle': {'llx': 33.4037, 'lly': 202.1591, 'urx': 228.6407, 'ury': 212.0591, 'height': 9.899994, 'width': 195.237}}, {'text': '❤ 如有任何疑问，请联系我司客服邮箱 cn.east.export@maersk.com', 'text_rectangle': {'llx': 30.9017, 'lly': 224.9531, 'urx': 302.7647, 'ury': 234.8531, 'height': 9.900009, 'width': 271.863}}, {'text': '航线:', 'text_rectangle': {'llx': 30.9017, 'lly': 247.7461, 'urx': 51.4037, 'ury': 257.6461, 'height': 9.899994, 'width': 20.502003}}, {'text': 'DRAGON ', 'text_rectangle': {'llx': 171.4997, 'lly': 247.7461, 'urx': 213.5027, 'ury': 257.6461, 'height': 9.899994, 'width': 42.003006}}, {'text': '开港参考时间(CY-Open):', 'text_rectangle': {'llx': 30.9017, 'lly': 259.1431, 'urx': 130.9097, 'ury': 269.0431, 'height': 9.899994, 'width': 100.007996}}, {'text': '请以码头网站时间为准', 'text_rectangle': {'llx': 168.4397, 'lly': 259.1431, 'urx': 258.4397, 'ury': 269.0431, 'height': 9.899994, 'width': 90.0}}, {'text': '截港时间(CY-CUTOFF):', 'text_rectangle': {'llx': 30.9017, 'lly': 270.5401, 'urx': 127.3907, 'ury': 280.4401, 'height': 9.899994, 'width': 96.489}}, {'text': 'TUE 08:00(请在此时间前完成集装箱进港并取得海关放行)', 'text_rectangle': {'llx': 134.8967, 'lly': 270.5401, 'urx': 363.9107, 'ury': 280.4401, 'height': 9.899994, 'width': 229.014}}, {'text': 'VGM截止时间(VGM Deadline):MON 17:00(请在此时间前提交VGM信息)', 'text_rectangle': {'llx': 30.9017, 'lly': 281.9371, 'urx': 316.9307, 'ury': 291.8371, 'height': 9.899994, 'width': 286.029}}, {'text': '样单截至参考时间(SI-CUT):', 'text_rectangle': {'llx': 30.9017, 'lly': 293.3331, 'urx': 141.3947, 'ury': 303.2331, 'height': 9.899994, 'width': 110.493}}, {'text': 'MON 00:00 (需要申报欧盟/美国及加拿大海关提前舱单的货物， ', 'text_rectangle': {'llx': 168.9167, 'lly': 293.3331, 'urx': 423.4367, 'ury': 303.2331, 'height': 9.899994, 'width': 254.52}}, {'text': '请严格按照此时间提交提单样本，及完整信息:包括所有品名的6位HS商品编码，托运人和收货人（或通知方）公司名称，街道地址，城', 'text_rectangle': {'llx': 30.9017, 'lly': 304.7301, 'urx': 563.9087, 'ury': 314.6301, 'height': 9.899994, 'width': 533.007}}, {'text': '市等)', 'text_rectangle': {'llx': 30.9017, 'lly': 316.1271, 'urx': 51.8987, 'ury': 326.0271, 'height': 9.899994, 'width': 20.997002}}, {'text': '* 以上信息供您参考, 如遇船舶操作时间变动,请以客户通知为准.', 'text_rectangle': {'llx': 30.9017, 'lly': 327.5241, 'urx': 280.9127, 'ury': 337.4241, 'height': 9.899994, 'width': 250.01099}}, {'text': '* 如预配船期中母船第一卸货港为亚洲中转港(如TPP), 样单截止参考时间为船开前发送即可.', 'text_rectangle': {'llx': 30.9017, 'lly': 338.9201, 'urx': 391.9097, 'ury': 348.8201, 'height': 9.899994, 'width': 361.008}}, {'text': '危险品上船表格截止时间:', 'text_rectangle': {'llx': 30.9017, 'lly': 350.3171, 'urx': 132.4037, 'ury': 360.2171, 'height': 9.899994, 'width': 101.502}}, {'text': 'Final MDGF cut off (for MSC only):', 'text_rectangle': {'llx': 30.9017, 'lly': 361.7141, 'urx': 169.4297, 'ury': 371.6141, 'height': 9.899994, 'width': 138.528}}, {'text': 'Final MDGF cut off (for ML and other VSA):FRI 11:00', 'text_rectangle': {'llx': 30.9017, 'lly': 373.1111, 'urx': 243.4817, 'ury': 383.0111, 'height': 9.899994, 'width': 212.58}}, {'text': 'Special remarks(注意事项):', 'text_rectangle': {'llx': 30.9017, 'lly': 395.9041, 'urx': 139.9097, 'ury': 405.8041, 'height': 9.899994, 'width': 109.007996}}, {'text': 'For DG Barge: 请在驳船开船前2个工作日的上午10:00前发送带有箱封号的Final DG ', 'text_rectangle': {'llx': 30.9017, 'lly': 407.3011, 'urx': 364.4507, 'ury': 417.2011, 'height': 9.900024, 'width': 333.549}}, {'text': 'form(并于MSDS合并为一个文件)到我司邮箱cn.east.export@maersk.com, 若Final DG form未能及时提供, 将可能影响箱子装船, ', 'text_rectangle': {'llx': 30.9017, 'lly': 418.6981, 'urx': 536.5847, 'ury': 428.5981, 'height': 9.900024, 'width': 505.683}}, {'text': '由此产生的相关费用需由Shipper承担, 请知悉.', 'text_rectangle': {'llx': 30.9017, 'lly': 430.0951, 'urx': 213.4217, 'ury': 439.9951, 'height': 9.899994, 'width': 182.52}}, {'text': '我们谨此通知您，对于所有出口至加蓬的货物，在提交提单补料时必须提供BIETC货物跟踪单号, ', 'text_rectangle': {'llx': 30.9017, 'lly': 452.8881, 'urx': 413.4107, 'ury': 462.7881, 'height': 9.899994, 'width': 382.509}}, {'text': '如未能及时提供跟踪单号，货物将无法装船， 缺失正确的BIETC货物跟踪单号可能存在货物到港后无法卸货， ', 'text_rectangle': {'llx': 30.9017, 'lly': 464.2851, 'urx': 467.4107, 'ury': 474.1851, 'height': 9.899994, 'width': 436.509}}, {'text': '原船回运以及海关罚款和扣货的风险。', 'text_rectangle': {'llx': 30.9017, 'lly': 475.6821, 'urx': 183.9017, 'ury': 485.5821, 'height': 9.899994, 'width': 153.0}}, {'text': 'We hereby inform you that all Cargo export to Gabon will require to submit BIETC Number on Shipping Instruction, Failure to comply ', 'text_rectangle': {'llx': 30.9017, 'lly': 487.0781, 'urx': 560.1017, 'ury': 496.9781, 'height': 9.899994, 'width': 529.2}}, {'text': 'with this requirements will lead to our inability to load your cargo at Origin Port, Load without BIETC number will result in possible ', 'text_rectangle': {'llx': 30.9017, 'lly': 498.4751, 'urx': 546.0887, 'ury': 508.3751, 'height': 9.899994, 'width': 515.187}}, {'text': 'retention on board, return back to origin, seizure of the cargo and settlement of fines before release.', 'text_rectangle': {'llx': 30.9017, 'lly': 509.8721, 'urx': 428.1077, 'ury': 519.7721, 'height': 9.899994, 'width': 397.206}}, {'text': '开顶柜,框架箱,超高超宽超长货物及冷冻柜:', 'text_rectangle': {'llx': 30.9017, 'lly': 532.6651, 'urx': 200.4077, 'ury': 542.5651, 'height': 9.900024, 'width': 169.506}}, {'text': '最晚提箱时间(for MSC only):', 'text_rectangle': {'llx': 30.9017, 'lly': 544.0621, 'urx': 145.4087, 'ury': 553.9621, 'height': 9.900024, 'width': 114.507}}, {'text': '最晚提箱时间(for ML and other VSA):FRI 11:00', 'text_rectangle': {'llx': 30.9017, 'lly': 555.4591, 'urx': 219.4607, 'ury': 565.3591, 'height': 9.899963, 'width': 188.55899}}, {'text': '* 超高超宽超长货物(OOG): ', 'text_rectangle': {'llx': 30.9017, 'lly': 578.2531, 'urx': 140.9087, 'ury': 588.1531, 'height': 9.899963, 'width': 110.007}}, {'text': '请在特种柜提柜截止时间前安排提柜，同时请确保在装箱后确认实际的货物尺寸和重量,如实际货物的超限尺寸和重量与订舱信息不符或', 'text_rectangle': {'llx': 30.9017, 'lly': 589.6491, 'urx': 564.4037, 'ury': 599.5491, 'height': 9.899963, 'width': 533.502}}, {'text': '者有任何尺寸和重量的更改申请,请在提箱后把柜号及其对应尺寸尽快发送到：CN.EAST.EXPORT@maersk.com ', 'text_rectangle': {'llx': 30.9017, 'lly': 601.0461, 'urx': 478.5617, 'ury': 610.9461, 'height': 9.900024, 'width': 447.66}}, {'text': '以避免后续更改可能导致货物被拒上船,由此产生的责任与费用将由客户全责承担. (SOC ', 'text_rectangle': {'llx': 30.9017, 'lly': 612.4431, 'urx': 378.4097, 'ury': 622.3431, 'height': 9.899963, 'width': 347.508}}, {'text': 'OOG-货主自有箱承载的超限货物也需在此时间前及时告知箱号及尺寸信息，如在订舱时已注明箱号并且箱号及对应尺寸没有更改的则', 'text_rectangle': {'llx': 30.9017, 'lly': 623.8401, 'urx': 558.9047, 'ury': 633.7401, 'height': 9.900024, 'width': 528.00305}}, {'text': '不需另外通知我司).', 'text_rectangle': {'llx': 30.9017, 'lly': 635.2361, 'urx': 108.4007, 'ury': 645.1361, 'height': 9.900024, 'width': 77.49901}}, {'text': '***危险品***', 'text_rectangle': {'llx': 30.9017, 'lly': 658.0301, 'urx': 78.9077, 'ury': 667.9301, 'height': 9.900024, 'width': 48.006}}, {'text': '1)如需更改危险品相关信息，请务必在箱子进港之前提出申请，任何更改都需要重新进行危险品审批，有可能会导致货物不能及时装船.', 'text_rectangle': {'llx': 30.9017, 'lly': 669.4271, 'urx': 563.4047, 'ury': 679.3271, 'height': 9.899963, 'width': 532.50305}}, {'text': '2)头程船涉及驳船运输的危险品，如需更改，请务必在箱子进港之前提出。另外请在驳船开船前2个工作日的上午10:00前发送带有箱封', 'text_rectangle': {'llx': 30.9017, 'lly': 680.8231, 'urx': 561.4247, 'ury': 690.7231, 'height': 9.899963, 'width': 530.523}}, {'text': '号的Final DG.', 'text_rectangle': {'llx': 30.9017, 'lly': 692.2201, 'urx': 86.9087, 'ury': 702.1201, 'height': 9.900024, 'width': 56.007}}, {'text': '3)请确保危险品货物的正确申报。重柜还场后凡发现危险品错报、瞒报、漏报的情况（包括但不止于更改货物属性，增加或修改UN号码', 'text_rectangle': {'llx': 30.9017, 'lly': 703.6171, 'urx': 564.8987, 'ury': 713.5171, 'height': 9.899963, 'width': 533.997}}, {'text': 'This document is subject to following:', 'text_rectangle': {'llx': 28.3502, 'lly': 716.767, 'urx': 128.8982, 'ury': 723.367, 'height': 6.5999756, 'width': 100.54799}}, {'text': '- This booking and carriage are subject to the Maersk Line Terms and Conditions of Carriage which are available upon request from the carrier or his representatives and are furthermore accessible on the', 'text_rectangle': {'llx': 28.3502, 'lly': 723.967, 'urx': 566.7482, 'ury': 730.567, 'height': 6.6000366, 'width': 538.398}}, {'text': 'Maersk Line website "<http://www.maerskline.com>" under "Services" / "General Business Terms".', 'text_rectangle': {'llx': 28.3502, 'lly': 731.167, 'urx': 291.8702, 'ury': 737.767, 'height': 6.6000366, 'width': 263.52002}}, {'text': '- The shipment is subject to tariff rates unless a correct and applicable service contract number is available', 'text_rectangle': {'llx': 28.3502, 'lly': 738.367, 'urx': 311.5382, 'ury': 744.967, 'height': 6.5999756, 'width': 283.18802}}, {'text': "- The carrier's right to substitute the named and/or performing vessel(s) with another vessel or vessels at any time.", 'text_rectangle': {'llx': 28.3502, 'lly': 745.567, 'urx': 334.5122, 'ury': 752.167, 'height': 6.5999756, 'width': 306.16202}}, {'text': '- Arrival, berthing, departure and transit times are estimated and given without guarantee and subject to change without prior notice', 'text_rectangle': {'llx': 28.3502, 'lly': 752.767, 'urx': 380.8502, 'ury': 759.367, 'height': 6.5999756, 'width': 352.5}}, {'text': '- All dates/times are given as reasonable estimates only and subject to change without prior notice.', 'text_rectangle': {'llx': 28.3502, 'lly': 759.967, 'urx': 292.1462, 'ury': 766.567, 'height': 6.6000366, 'width': 263.79602}}, {'text': 'Shipments destined for or carried/transhipped via the USA:', 'text_rectangle': {'llx': 28.3502, 'lly': 767.167, 'urx': 184.5602, 'ury': 773.767, 'height': 6.6000366, 'width': 156.20999}}, {'text': '- This document is given subject to the customer providing the correct cargo description in accordance with U.S. law, including U.S. Customs requirements as described in Customs Rules and Regulations,', 'text_rectangle': {'llx': 28.3502, 'lly': 774.367, 'urx': 565.7582, 'ury': 780.967, 'height': 6.5999756, 'width': 537.40796}}, {'text': '19 CFR Parts 4, 113 and 178 of October 31, 2002', 'text_rectangle': {'llx': 28.3502, 'lly': 781.567, 'urx': 159.3542, 'ury': 788.167, 'height': 6.5999756, 'width': 131.004}}, {'text': 'Page', 'text_rectangle': {'llx': 249.255, 'lly': 789.303, 'urx': 266.311, 'ury': 798.103, 'height': 8.800049, 'width': 17.056}}, {'text': '1/3', 'text_rectangle': {'llx': 276.556, 'lly': 789.359, 'urx': 288.348, 'ury': 798.159, 'height': 8.799988, 'width': 11.791992}}], 'cells_text': []}
#     # scale=4.1663
#     # file_name="2028230"
#     # page_id=1
#     # generator_data_from_origin_text(origin_text,scale,file_name,page_id)
#     draw_rect_to_img("/data-hdd/LingYue/ocr_data/2028230_1.txt","/data-hdd/LingYue/booking_confirmation/base_class/maersk/doc2img/12028230.jpg","/data-hdd/LingYue/ocr_data/img")





def read_vocab(vocab_dir):
    """读取词汇表"""
    with open(vocab_dir, "r", encoding="utf-8") as fp:
        words = [_.strip() for _ in fp.readlines()]
    word_to_id = dict(zip(words, range(len(words))))
    return words, word_to_id


def read_vec_file(filename):
    chars = []
    vecs = []
    f = open(filename, "r", encoding="utf-8")
    lines = [line.rstrip() for line in f.readlines()]
    for line in lines:
        char, vec = line.split("\t")
        chars.append(char)
        vecs.append(vec.split(" "))
    return chars, vecs


def word_to_vocab_id(s, word_to_id, max_length):
    ids = []
    for char in s:
        try:
            ids.append(word_to_id[char])
        except:
            ids.append(word_to_id["<UNK>"])
    assert len(ids) == len(s)
    if len(ids) >= max_length:
        ids = ids[:max_length]
    else:
        ids = ids + [word_to_id["<PAD>"]] * (max_length - len(ids))
    return ids



class Singleton(object):
    def __init__(self, cls):
        self._cls = cls
        self._instance = {}

    def __call__(self, **kwargs):
        if self._cls not in self._instance:
            self._instance[self._cls] = self._cls(**kwargs)
        return self._instance[self._cls]



from abc import abstractmethod

from torch import nn


class BaseModel(nn.Module):

    def __init__(self, config):
        super(BaseModel, self).__init__()
        self.config = config

    @abstractmethod
    def get_default_input(self):
        """
        获取input shape 大小
        :return:
        """
        pass



import math
import os
import pathlib
import shutil
import sys
from abc import abstractmethod

import torch
import torchvision

from yqn_common import metric_logger as utils
from yqn_config.base_config import BaseConfig
from yqn_pytorch_framework.device import tensor_load_device


def save_checkpoint(state, is_best, directory, filename='checkpoint.pth'):
    pathlib.Path(directory).mkdir(parents=True, exist_ok=True)
    torch.save(state, os.path.join(directory, filename))
    shutil.copyfile(os.path.join(directory, filename), os.path.join(directory, 'model_last.pth'))
    if is_best:
        shutil.copyfile(os.path.join(directory, filename), os.path.join(directory, 'model_best.pth'))


class BaseModelEngine:
    def __init__(self, config: BaseConfig):
        super(BaseModelEngine, self).__init__()
        self.config = config

    @abstractmethod
    def criterion(self, outputs, labels):
        pass

    @abstractmethod
    def accuracy(self, outputs, labels):
        pass

    @abstractmethod
    def load_inputs(self, dataset):
        pass

    @abstractmethod
    def load_labels(self, dataset):
        pass

    @staticmethod
    def check_best(last_best_acc, focal_acc):
        is_best = True
        if last_best_acc is None:
            return is_best
        else:
            for index in range(len(focal_acc)):
                is_best = is_best & torch.gt(focal_acc[index], last_best_acc[index]).item()
            return is_best

    @staticmethod
    def get_save_info_per_epoch(epoch, step, model, optimizer, acc_list):
        return {
            'epoch': epoch + 1,
            'step': step,
            'state_dict': model.state_dict(),
            'acc_list': acc_list,
            'optimizer': optimizer.state_dict(),
        }

    def write_scalar(self, writer, focal_loss, focal_acc, step, epoch_index):
        if (epoch_index + 1) % self.config.summary_freq == 0:
            if focal_loss:
                for index, loss_value in enumerate(focal_loss):
                    writer.add_scalar('Train/loss_' + str(index),
                                      loss_value,
                                      step)
            if focal_acc:
                for index, acc_value in enumerate(focal_acc):
                    writer.add_scalar('Train/acc_' + str(index),
                                      acc_value,
                                      step)

    def write_image(self, writer, labels, predicts, step, epoch_index):
        if (epoch_index + 1) % self.config.summary_freq == 0:
            targets_img_grid = torchvision.utils.make_grid(labels[0, :, :], nrow=2)
            writer.add_image('Image_Label/targets_images', targets_img_grid, step)
            predicts_img_grid = torchvision.utils.make_grid(predicts[0, :, :], nrow=2)
            writer.add_image('Image_Pred/predicts_images', predicts_img_grid, step)

    def save_model(self, epoch, is_best, focal_acc, model, optimizer, step):
        if step % self.config.save_freq == 0:
            save_info = self.get_save_info_per_epoch(epoch, step, model, optimizer, focal_acc)
            save_checkpoint(save_info,
                            is_best,
                            self.config.get_train_model_out_dir(self.config.split_date),
                            filename='checkpoint_%d.pth' % step)

    def train_one_epoch(self,
                        model,
                        optimizer,
                        data_loader,
                        device,
                        epoch,
                        lr_scheduler=None,
                        last_best_acc=None,
                        step=0,
                        writer=None):
        current_acc = None
        try:
            metric_logger = utils.MetricLogger(delimiter="  ")
            metric_logger.add_meter('lr', utils.SmoothedValue(window_size=1, fmt='{value:.6f}'))
            header = 'Epoch: [{}]'.format(epoch)
            model.train()
            for dataset, index in metric_logger.log_every(data_loader, self.config.progress_freq, header):
                inputs_dataset = self.load_inputs(dataset)
                labels_dataset = self.load_labels(dataset)
                input_features = []
                labels = []
                if type(inputs_dataset) is list or type(inputs_dataset) is tuple:
                    for input_dataset in inputs_dataset:
                        input_features.append(tensor_load_device(input_dataset, device))
                else:
                    input_features = tensor_load_device(inputs_dataset, device)
                if type(labels_dataset) is list or type(labels_dataset) is tuple:
                    for label_dataset in labels_dataset:
                        labels.append(tensor_load_device(label_dataset, device))
                else:
                    labels = tensor_load_device(labels_dataset, device)
                outputs = model(*input_features)
                if len(outputs) == 1:
                    outputs = outputs[0]
                focal_loss = self.criterion(outputs=outputs, labels=labels)
                focal_acc = self.accuracy(outputs=outputs, labels=labels)
                # focal_loss = self.criterion(outputs=tuple(outputs), labels=tuple(labels))
                # focal_acc = self.accuracy(outputs=tuple(outputs), labels=tuple(labels))
                current_acc = focal_acc
                if not math.isfinite(focal_loss[0]):
                    print("Loss is {}, stopping training".format(focal_loss[0]))
                    sys.exit(1)

                optimizer.zero_grad()
                focal_loss[0].backward()
                optimizer.step()
                if lr_scheduler:
                    lr_scheduler.step()
                step += 1

                is_best = self.check_best(last_best_acc, focal_acc)
                if is_best:
                    last_best_acc = focal_acc
                self.save_model(epoch, is_best, focal_acc, model, optimizer, step)
                if writer is not None:
                    if self.config.write_scalar:
                        self.write_scalar(writer, focal_loss, focal_acc, step, index)
                    if self.config.write_image:
                        self.write_image(writer, labels, outputs, step, index)
                    writer.flush()
                metric_logger.update(loss=focal_loss[0], lr=optimizer.param_groups[0]["lr"])
        except KeyboardInterrupt:
            pass
            # save_info = self.get_save_info_per_epoch(epoch, step, model, optimizer, current_acc)
            # torch.save(save_info,
            #            os.path.join(self.config.get_train_model_out_dir(self.config.split_date), 'model_last.pth'))

        return last_best_acc, step



"""
    FILE :  CRF.py
    FUNCTION : None
    REFERENCE : https://github.com/jiesutd/NCRFpp/blob/master/model/crf.py
"""
import torch
from torch.autograd.variable import Variable
import torch.nn as nn


def log_sum_exp(vec, m_size):
    """
    Args:
        vec: size=(batch_size, vanishing_dim, hidden_dim)
        m_size: hidden_dim

    Returns:
        size=(batch_size, hidden_dim)
    """
    _, idx = torch.max(vec, 1)  # B * 1 * M
    batch_size = vec.size(0)
    max_score = torch.gather(vec, 1, idx.view(batch_size, 1, m_size)).view(batch_size, 1, m_size)  # B * M
    return max_score.view(batch_size, m_size) + torch.log(torch.sum(
        torch.exp(vec - max_score.expand_as(vec)), 1)).view(batch_size, m_size)


class CRF(nn.Module):
    """
        CRF
    """

    def __init__(self, **kwargs):
        """
        kwargs:
            target_size: int, target size
            device: str, device
        """
        super(CRF, self).__init__()
        for k in kwargs:
            self.__setattr__(k, kwargs[k])
        device = self.device

        # init transitions
        self.START_TAG, self.STOP_TAG = -2, -1
        init_transitions = torch.zeros(self.target_size + 2, self.target_size + 2, device=device)
        init_transitions[:, self.START_TAG] = -10000.0
        init_transitions[self.STOP_TAG, :] = -10000.0
        self.transitions = nn.Parameter(init_transitions)

    def _forward_alg(self, feats, mask):
        """
        Do the forward algorithm to compute the partition function (batched).

        Args:
            feats: size=(batch_size, seq_len, self.target_size+2)
            mask: size=(batch_size, seq_len)

        Returns:
            xxx
        """
        batch_size = feats.size(0)
        seq_len = feats.size(1)
        tag_size = feats.size(2)
        mask = mask.transpose(1, 0).contiguous()
        ins_num = seq_len * batch_size
        """ be careful the view shape, it is .view(ins_num, 1, tag_size) but not .view(ins_num, tag_size, 1) """
        feats = feats.transpose(1, 0).contiguous().view(ins_num, 1, tag_size).expand(ins_num, tag_size, tag_size)
        """ need to consider start """
        scores = feats + self.transitions.view(1, tag_size, tag_size).expand(ins_num, tag_size, tag_size)
        scores = scores.view(seq_len, batch_size, tag_size, tag_size)
        # build iter
        seq_iter = enumerate(scores)
        _, inivalues = next(seq_iter)  # bat_size * from_target_size * to_target_size
        """ only need start from start_tag """
        partition = inivalues[:, self.START_TAG, :].clone().view(batch_size, tag_size, 1)  # bat_size * to_target_size

        """
        add start score (from start to all tag, duplicate to batch_size)
        partition = partition + self.transitions[START_TAG,:].view(1, tag_size, 1).expand(batch_size, tag_size, 1)
        iter over last scores
        """
        for idx, cur_values in seq_iter:
            """
            previous to_target is current from_target
            partition: previous results log(exp(from_target)), #(batch_size * from_target)
            cur_values: bat_size * from_target * to_target
            """
            cur_values = cur_values + partition.contiguous().view(batch_size, tag_size, 1).expand(batch_size, tag_size,
                                                                                                  tag_size)
            cur_partition = log_sum_exp(cur_values, tag_size)

            mask_idx = mask[idx, :].view(batch_size, 1).expand(batch_size, tag_size)

            """ effective updated partition part, only keep the partition value of mask value = 1 """
            masked_cur_partition = cur_partition.masked_select(mask_idx)
            """ let mask_idx broadcastable, to disable warning """
            mask_idx = mask_idx.contiguous().view(batch_size, tag_size, 1)

            """ replace the partition where the maskvalue=1, other partition value keeps the same """
            partition.masked_scatter_(mask_idx, masked_cur_partition)
        """ 
        until the last state, add transition score for all partition (and do log_sum_exp) 
        then select the value in STOP_TAG 
        """
        cur_values = (self.transitions
                      .view(1, tag_size, tag_size)
                      .expand(batch_size, tag_size, tag_size)
                      + partition.contiguous()
                      .view(batch_size, tag_size, 1)
                      .expand(batch_size, tag_size, tag_size))
        cur_partition = log_sum_exp(cur_values, tag_size)
        final_partition = cur_partition[:, self.STOP_TAG]
        return final_partition.sum(), scores

    def _viterbi_decode(self, feats, mask):
        """
            input:
                feats: (batch, seq_len, self.tag_size+2)
                mask: (batch, seq_len)
            output:
                decode_idx: (batch, seq_len) decoded sequence
                path_score: (batch, 1) corresponding score for each sequence (to be implementated)
        """
        # print(feats.size())
        batch_size = feats.size(0)
        seq_len = feats.size(1)
        tag_size = feats.size(2)
        # assert(tag_size == self.tagset_size+2)
        """ calculate sentence length for each sentence """
        length_mask = torch.sum(mask.long(), dim=1).view(batch_size, 1).long()
        """ mask to (seq_len, batch_size) """
        mask = mask.transpose(1, 0).contiguous()
        ins_num = seq_len * batch_size
        """ be careful the view shape, it is .view(ins_num, 1, tag_size) but not .view(ins_num, tag_size, 1) """
        feats = feats.transpose(1, 0).contiguous().view(ins_num, 1, tag_size).expand(ins_num, tag_size, tag_size)
        """ need to consider start """
        scores = feats + self.transitions.view(1, tag_size, tag_size).expand(ins_num, tag_size, tag_size)
        scores = scores.view(seq_len, batch_size, tag_size, tag_size)

        # build iter
        seq_iter = enumerate(scores)
        # record the position of best score
        back_points = list()
        partition_history = list()
        ##  reverse mask (bug for mask = 1- mask, use this as alternative choice)
        # mask = 1 + (-1)*mask
        mask = (1 - mask.long()).byte()
        _, inivalues = next(seq_iter)  # bat_size * from_target_size * to_target_size
        """ only need start from start_tag """
        partition = inivalues[:, self.START_TAG, :].clone().view(batch_size, tag_size)  # bat_size * to_target_size
        partition_history.append(partition)
        # iter over last scores
        for idx, cur_values in seq_iter:
            """
            previous to_target is current from_target
            partition: previous results log(exp(from_target)), #(batch_size * from_target)
            cur_values: batch_size * from_target * to_target
            """
            cur_values = cur_values + partition.contiguous().view(batch_size, tag_size, 1).expand(batch_size, tag_size,
                                                                                                  tag_size)
            """ forscores, cur_bp = torch.max(cur_values[:,:-2,:], 1) # do not consider START_TAG/STOP_TAG """
            partition, cur_bp = torch.max(cur_values, 1)
            partition_history.append(partition)
            """
            cur_bp: (batch_size, tag_size) max source score position in current tag
            set padded label as 0, which will be filtered in post processing
            """
            cur_bp.masked_fill_(mask[idx].view(batch_size, 1).expand(batch_size, tag_size).bool(), 0)
            back_points.append(cur_bp)
        """ add score to final STOP_TAG """
        partition_history = (torch.cat(partition_history, 0)
                             .view(seq_len, batch_size, -1)
                             .transpose(1, 0)
                             .contiguous())  ## (batch_size, seq_len. tag_size)
        """ get the last position for each setences, and select the last partitions using gather() """
        last_position = length_mask.view(batch_size, 1, 1).expand(batch_size, 1, tag_size) - 1
        last_partition = torch.gather(partition_history, 1, last_position).view(batch_size, tag_size, 1)
        """ calculate the score from last partition to end state (and then select the STOP_TAG from it) """
        last_values = last_partition.expand(batch_size, tag_size, tag_size) + self.transitions.view(1, tag_size,
                                                                                                    tag_size).expand(
            batch_size, tag_size, tag_size)
        _, last_bp = torch.max(last_values, 1)
        pad_zero = torch.zeros(batch_size, tag_size, device=self.device, requires_grad=True).long()
        back_points.append(pad_zero)
        back_points = torch.cat(back_points).view(seq_len, batch_size, tag_size)

        """ elect end ids in STOP_TAG """
        pointer = last_bp[:, self.STOP_TAG]
        insert_last = pointer.contiguous().view(batch_size, 1, 1).expand(batch_size, 1, tag_size)
        back_points = back_points.transpose(1, 0).contiguous()
        """move the end ids(expand to tag_size) to the corresponding position of back_points to replace the 0 values """
        back_points.scatter_(1, last_position, insert_last)
        back_points = back_points.transpose(1, 0).contiguous()
        """ decode from the end, padded position ids are 0, which will be filtered if following evaluation """
        # decode_idx = Variable(torch.LongTensor(seq_len, batch_size))
        decode_idx = torch.empty(seq_len, batch_size, device=self.device, requires_grad=True).long()
        decode_idx[-1] = pointer.detach()
        for idx in range(len(back_points) - 2, -1, -1):
            pointer = torch.gather(back_points[idx], 1, pointer.contiguous().view(batch_size, 1))
            decode_idx[idx] = pointer.detach().view(batch_size)
        path_score = None
        decode_idx = decode_idx.transpose(1, 0)
        return path_score, decode_idx

    def forward(self, feats, mask):
        """
        :param feats:
        :param mask:
        :return:
        """
        path_score, best_path = self._viterbi_decode(feats, mask)
        return path_score, best_path

    def _score_sentence(self, scores, mask, tags):
        """
        Args:
            scores: size=(seq_len, batch_size, tag_size, tag_size)
            mask: size=(batch_size, seq_len)
            tags: size=(batch_size, seq_len)

        Returns:
            score:
        """
        # print(scores.size())
        batch_size = scores.size(1)
        seq_len = scores.size(0)
        tag_size = scores.size(-1)
        tags = tags.view(batch_size, seq_len)
        """ convert tag value into a new format, recorded label bigram information to index """
        # new_tags = Variable(torch.LongTensor(batch_size, seq_len))
        new_tags = torch.empty(batch_size, seq_len, device=self.device, requires_grad=True).long()
        for idx in range(seq_len):
            if idx == 0:
                new_tags[:, 0] = (tag_size - 2) * tag_size + tags[:, 0]
            else:
                new_tags[:, idx] = tags[:, idx - 1] * tag_size + tags[:, idx]

        """ transition for label to STOP_TAG """
        end_transition = self.transitions[:, self.STOP_TAG].contiguous().view(1, tag_size).expand(batch_size, tag_size)
        """ length for batch,  last word position = length - 1 """
        length_mask = torch.sum(mask, dim=1).view(batch_size, 1).long()
        """ index the label id of last word """
        end_ids = torch.gather(tags, 1, length_mask - 1)

        """ index the transition score for end_id to STOP_TAG """
        end_energy = torch.gather(end_transition, 1, end_ids)

        """ convert tag as (seq_len, batch_size, 1) """
        new_tags = new_tags.transpose(1, 0).contiguous().view(seq_len, batch_size, 1)
        """ need convert tags id to search from 400 positions of scores """
        tg_energy = torch.gather(scores.view(seq_len, batch_size, -1), 2, new_tags).view(seq_len, batch_size)
        tg_energy = tg_energy.masked_select(mask.transpose(1, 0))

        """
        add all score together
        gold_score = start_energy.sum() + tg_energy.sum() + end_energy.sum()
        """
        gold_score = tg_energy.sum() + end_energy.sum()

        return gold_score

    def neg_log_likelihood_loss(self, feats, mask, tags):
        """
        Args:
            feats: size=(batch_size, seq_len, tag_size)
            mask: size=(batch_size, seq_len)
            tags: size=(batch_size, seq_len)
        """
        batch_size = feats.size(0)
        forward_score, scores = self._forward_alg(feats, mask)
        gold_score = self._score_sentence(scores, mask, tags)
        return forward_score - gold_score



import hashlib
import os
import shutil
import tempfile
from urllib.request import urlopen

from yqn_common.oss_func import oss_calculate_file_crc64


def download_url_to_file(url, dst, crc=None):
    file_size = None
    # We use a different API for python2 since urllib(2) doesn't recognize the CA
    # certificates in older Python
    u = urlopen(url)
    meta = u.info()
    if hasattr(meta, 'getheaders'):
        content_length = meta.getheaders("Content-Length")
    else:
        content_length = meta.get_all("Content-Length")
    if content_length is not None and len(content_length) > 0:
        file_size = int(content_length[0])

    dst = os.path.expanduser(dst)
    dst_dir = os.path.dirname(dst)
    f = tempfile.NamedTemporaryFile(delete=False, dir=dst_dir)

    try:
        while True:
            buffer = u.read(8192)
            if len(buffer) == 0:
                break
            f.write(buffer)
        f.close()
        if crc is not None:
            file_crc = oss_calculate_file_crc64(f.name)
            if str(file_crc) != crc:
                raise RuntimeError('invalid hash value (expected "{}", got "{}")'
                                   .format(crc, file_crc))
        shutil.move(f.name, dst)
    finally:
        f.close()
        if os.path.exists(f.name):
            os.remove(f.name)



import csv
import re

import jieba


def get_jieba_tokenizer(user_dict_path):
    jieba_tokenizer_name = jieba.Tokenizer()
    # jieba.initialize()
    dics = csv.reader(open(user_dict_path, 'r', encoding='utf8'))
    print("load jieba once ")
    for row in dics:
        if len(row) == 2:
            line = row[0]
            line = re.sub(r'[ ]+', '', str(line))
            line = re.sub("＆", '&', re.sub('）', ')', re.sub('（', '(', line)))
            line = re.sub("：", ":", line)
            line1 = re.sub("\✤+|'|’|;|\xa0|\s|\t|\n|' '|✣|,|，|◇|\.|\x7f|\)|\(|:", '', line)
            if len(line) >= 4:
                line1 = line1
            else:
                line1 = line
            # jieba.add_word(line.strip().upper(), tag=row[1].strip())
            # jieba.suggest_freq(line.strip())
            # jieba.add_word(line1.strip().upper(), tag=row[1].strip())
            # jieba.suggest_freq(line1.strip())
            jieba_tokenizer_name.add_word(line.strip().upper(), tag=row[1].strip())
            jieba_tokenizer_name.suggest_freq(line.strip())
            jieba_tokenizer_name.add_word(line1.strip().upper(), tag=row[1].strip())
            jieba_tokenizer_name.suggest_freq(line1.strip())
            jieba_tokenizer_name.add_word(line1.strip().upper() + ":", tag=row[1].strip())
            jieba_tokenizer_name.suggest_freq(line1.strip())


    return jieba_tokenizer_name

def get_jieba_tokenizer_file(user_dict_path):
    jieba_tokenizer_name = jieba.Tokenizer()
    # jieba.initialize()
    jieba_tokenizer_name.load_userdict(user_dict_path)
    return jieba_tokenizer_name


import argparse
import logging
import os
from collections import OrderedDict
from datetime import datetime

from yqn_common.logger import NTLogger


class MyFormatter(logging.Formatter):
    converter = datetime.fromtimestamp

    def formatTime(self, record, datefmt=None):
        ct = self.converter(record.created)
        if datefmt:
            s = ct.strftime(datefmt)
        else:
            t = ct.strftime("%Y-%m-%d %H:%M:%S")
            s = "%s.%03d" % (t, record.msecs)
        return s


def set_logger(context, verbose=False):
    if os.name == 'nt':  # for Windows
        return NTLogger(context, verbose)

    logger = logging.getLogger(context)
    logger.setLevel(logging.DEBUG if verbose else logging.WARNING)
    formatter = MyFormatter(
        fmt='%(asctime)s;%(levelname)-.1s:' + context + ':\t[%(filename).25s:%(funcName).12s:%(lineno)4d]:%(message)s')
    console_handler = logging.StreamHandler()
    console_handler.setLevel(logging.DEBUG if verbose else logging.INFO)
    console_handler.setFormatter(formatter)
    logger.handlers = []
    logger.addHandler(console_handler)
    return logger


class CappedHistogram:
    """Space capped dict with aggregate stat tracking.

    Evicts using LRU policy when at capacity; evicted elements are added to aggregate stats.
    Arguments:
    capacity -- the capacity limit of the dict
    """

    def __init__(self, capacity):
        self.cache = OrderedDict()
        self.capacity = capacity
        self.base_bins = 0
        self.base_count = 0
        self.base_min = float('inf')
        self.min_count = 0
        self.base_max = 0
        self.max_count = 0

    def __getitem__(self, key):
        if key in self.cache:
            return self.cache[key]
        return 0

    def __setitem__(self, key, value):
        if key in self.cache:
            del self.cache[key]
        self.cache[key] = value
        if len(self.cache) > self.capacity:
            self._evict()

    def total_size(self):
        return self.base_bins + len(self.cache)

    def __len__(self):
        return len(self.cache)

    def values(self):
        return self.cache.values()

    def _evict(self):
        key, val = self.cache.popitem(False)
        self.base_bins += 1
        self.base_count += val
        if val < self.base_min:
            self.base_min = val
            self.min_count = 1
        elif val == self.base_min:
            self.min_count += 1
        if val > self.base_max:
            self.base_max = val
            self.max_count = 1
        elif val == self.base_max:
            self.max_count += 1

    def get_stat_map(self, name):
        if len(self.cache) == 0:
            return {}
        counts = self.cache.values()
        avg = (self.base_count + sum(counts)) / (self.base_bins + len(counts))
        min_, max_ = min(counts), max(counts)
        num_min, num_max = 0, 0
        if self.base_min <= min_:
            min_ = self.base_min
            num_min += self.min_count
        if self.base_min >= min_:
            num_min += sum(v == min_ for v in counts)

        if self.base_max >= max_:
            max_ = self.base_max
            num_max += self.max_count
        if self.base_max <= max_:
            num_max += sum(v == max_ for v in counts)

        return {
            'avg_%s' % name: avg,
            'min_%s' % name: min_,
            'max_%s' % name: max_,
            'num_min_%s' % name: num_min,
            'num_max_%s' % name: num_max,
        }



import os
import shutil
from abc import abstractmethod

import torch
from torch.backends import cudnn
from torch.utils.data.dataloader import DataLoader
from torch.utils.tensorboard import SummaryWriter
from torch import optim

from yqn_config.base_config import BaseConfig
from yqn_pytorch_framework.device import get_cpu_device, tensor_load_device
from yqn_pytorch_framework.train.base_engine import BaseModelEngine


class BaseModelTrain:

    def __init__(self, engine: BaseModelEngine):
        self.engine = engine
        super(BaseModelTrain, self).__init__()

    @abstractmethod
    def load_model(self, model_config: BaseConfig):
        """
        :param model_config:
        :return:  nn model
        """
        pass

    @abstractmethod
    def load_data(self, config: BaseConfig, data_flag='train'):
        """
        加载训练数据
        :param config:
        :param data_flag
        :return:
        """
        pass

    @staticmethod
    def get_resume_file(config: BaseConfig):
        """
        获取继续训练的模型位置
        :param config:
        :return:
        """
        directory = config.get_train_model_out_dir(config.split_date)
        resume_file = os.path.join(directory, 'model_last.pth')
        return resume_file

    @staticmethod
    def get_optimizer(model, config: BaseConfig):
        """
        获取loss Optimizer 方法
        :param self:
        :param model:
        :param config:
        :return:
        """
        if config.optimizer in ["sdg", "adam"]:
            if config.optimizer == "sdg":
                return torch.optim.SGD(model.parameters(), lr=config.learning_rate)
            else:
                return torch.optim.Adam(model.parameters(), lr=config.learning_rate)
        else:
            return torch.optim.Adam(model.parameters(), lr=config.learning_rate)

    def start_train(self, model, config: BaseConfig):
        train_data = self.load_data(config, 'train')
        train_loader = DataLoader(dataset=train_data,
                                  batch_size=config.batch_size,
                                  shuffle=config.shuffle_batch,
                                  # num_workers=8,
                                  pin_memory=True)
        writer = SummaryWriter(config.get_train_log_dir())
        model = tensor_load_device(model, config.device)
        dummy_input = model.get_default_input()
        writer.add_graph(model, input_to_model=dummy_input)
        writer.flush()
        # optimizer = self.get_optimizer(model=model, config=config)
        optimizer = optim.Adam(model.parameters(), lr=config.learning_rate, amsgrad=True,
                               weight_decay=config.lr_rate_decay)
        for group in optimizer.param_groups:
            group.setdefault('initial_lr', group['lr'])

        lr_scheduler = None
        best_acc = None
        last_epoch = 0
        global_step = 0
        if not config.retrain:
            resume_file = self.get_resume_file(config)
            if os.path.exists(resume_file):
                print("=> loading checkpoint '{}'".format(resume_file))
                checkpoint = torch.load(resume_file, map_location=device)
                last_epoch = checkpoint['epoch']
                global_step = checkpoint['step']
                if 'best_acc_list' in checkpoint:
                    best_acc = checkpoint['best_acc_list']
                model.load_state_dict(checkpoint['state_dict'])
                print("=> loaded checkpoint '{}' (epoch {} global step {})".format(resume_file,
                                                                                   checkpoint['epoch'],
                                                                                   checkpoint['step']))
        else:
            directory = config.get_train_model_out_dir(config.split_date)
            shutil.rmtree(directory)
        if lr_scheduler:
            lr_scheduler.step(epoch=global_step)
        for epoch in range(last_epoch, last_epoch + config.num_epochs):
            best_acc, global_step = self.engine.train_one_epoch(model,
                                                                optimizer=optimizer,
                                                                data_loader=train_loader,
                                                                device=config.device,
                                                                epoch=epoch,
                                                                lr_scheduler=lr_scheduler,
                                                                step=global_step,
                                                                last_best_acc=best_acc,
                                                                writer=writer)

    def train(self, config):
        model = self.load_model(model_config=config)
        model = tensor_load_device(model, config.device)
        print(model)
        self.start_train(model, config)



from torch import nn

from yqn_pytorch_framework.layer.pytorch_textcnn_conv import TextCNNConv


class TextCNNConvPoolBlock(nn.Module):
    """
    Convolution Block
    """

    def __init__(self, sentence_length, out_ch, kernel_size):
        super(TextCNNConvPoolBlock, self).__init__()
        max_pool_kernel = sentence_length - kernel_size + 1
        self.conv = TextCNNConv(sentence_length, out_ch, kernel_size)
        self.max_pool = nn.MaxPool1d(kernel_size=max_pool_kernel)

    def forward(self, x):
        _out = self.conv(x)
        _out_pool = self.max_pool(_out)
        return _out_pool



'''
生成NER标注前的文件
输入： 文件（解析成图片和文本块之后）， 需要标注的标签类型， classes , annotations
输出： 符合工具模式的json文件（且已经预标注）
'''
import json
import os


def in_which_block(e, block_len):
    word_end = e["end"]
    block_length = list(block_len.values())
    block_id = -1
    for i in range(len(block_length) - 1):
        if word_end >= block_length[i] and word_end < block_length[i + 1]:
            block_id = i + 1
            break
    if block_id > 0:
        e["start"] = e["start"] - block_length[block_id - 1]
        e["end"] = e["end"] - block_length[block_id - 1]
    return e, block_id


def gen_json(filename, contents, keys_nlp, classes, annotations, save_path):
    json_info = {"file_id": filename,
                 "classes": classes,
                 "annotations": annotations,
                 "labels": []}
    # pre_entities 的每个元素是每一块的预标注内容
    # 先把内容做成一整个文档，为什么不分块？因为现在NER没有分块
    predict_class = ""
    real_class = ""
    state = True
    block_text = {}
    min_x = contents[0]["text_rectangle"]["llx"]
    min_y = contents[0]["text_rectangle"]["lly"]
    max_x = contents[-1]["text_rectangle"]["urx"]
    max_y = contents[-1]["text_rectangle"]["ury"]
    # 有block的范围吗？
    for i in range(len(contents)):
        block_id = contents[i]["block_index"]
        ttext = contents[i]["text"]
        if block_id not in block_text:
            block_text[block_id] = ttext.split("#")[0]
        else:
            block_text[block_id] += ttext.split("#")[0]
    block_len = {}
    for k, v in block_text.items():
        if k == 0:
            block_len[k] = len(v)
        else:
            block_len[k] = len(v) + block_len[k - 1]
    print("block_len", block_len)
    block_anno = {}
    for k, v in block_text.items():
        block_anno[k] = []

    position = {"left": min_x,
                "top": min_y,
                "right": max_x,
                "bottom": max_y}
    # anno = []
    for e in keys_nlp:
        e, block_id = in_which_block(e, block_len)
        block_anno[block_id].append({"start": e["start"],
                                     "end": e["end"],
                                     "annotation": e["type"].split("#")[0]})
    for k, v in block_text.items():
        json_info["labels"].append({"index": k,
                                    "text": v,
                                    "predict_class": predict_class,
                                    "real_class": real_class,
                                    "state": state,
                                    "position": position,
                                    "annotations": block_anno[k]})
    # json_info["labels"].append({"index": index,
    #                             "text": text,
    #                             "predict_class": predict_class,
    #                             "real_class": real_class,
    #                             "state": state,
    #                             "position": position,
    #                             "annotations": anno})
    json_info_string = json.dumps(json_info, ensure_ascii=False)
    f = open(os.path.join(save_path, str(filename) + ".json"), "w", encoding="utf-8")
    f.write(json_info_string)
    f.close()

    return



import torch

from yqn_pytorch_framework.device import tensor_load_device


def pytorch_to_onnx(model, device, pth_path, save_path, dummy_inputs, input_names, output_names):
    model = tensor_load_device(model, device).eval()
    checkpoint = torch.load(pth_path, map_location=device)
    model.load_state_dict(checkpoint['state_dict'])
    model.eval()
    dynamic_dict = {}
    # for input_name in input_names:
    #     dynamic_dict[input_name] = {0: "batch_size"}
    # for output_name in output_names:
    #     dynamic_dict[output_name] = {0: "batch_size"}
    torch.onnx.export(model,
                      dummy_inputs,
                      save_path,
                      input_names=input_names,
                      output_names=output_names,
                      export_params=True,
                      # dynamic_axes=dynamic_dict,
                      verbose=True)



import os
from abc import abstractmethod

import torch
import numpy as np
from yqn_config.base_config import BaseConfig
from yqn_exception.exception import InputSizeException, InferModelException
from yqn_pytorch_framework.device import get_device, tensor_load_device
from yqn_pytorch_framework.train.base_engine import BaseModelEngine


class BaseModelInfer:
    def __init__(self, model_config: BaseConfig):
        self.config = model_config
        self.device = model_config.device
        with torch.no_grad():
            self.model = self.load_model().cpu()
            resume_file = self.get_infer_model_file()
            if os.path.exists(resume_file):
                print("=> loading checkpoint '{}'".format(resume_file))
                checkpoint = torch.load(resume_file, map_location=torch.device('cpu'))
                if hasattr(self.config, "saved_state"):
                    if self.config.saved_state:
                        self.model.load_state_dict(checkpoint, strict=False)
                    else:
                        self.model.load_state_dict(checkpoint['state_dict'])
                else:
                    self.model.load_state_dict(checkpoint['state_dict'])

                self.model = tensor_load_device(self.model, self.device)

                self.model.eval()
                print("=> loaded checkpoint '{}' ".format(resume_file))
            else:
                raise InferModelException("infer model not found " + str(resume_file))

        super(BaseModelInfer, self).__init__()

    def get_infer_model_file(self):
        """
        :return: infer_model 文件路径
        """
        directory = self.config.get_train_model_out_dir(self.config.split_date)
        infer_model_path = os.path.join(directory, 'model_best.pth')
        return infer_model_path

    @abstractmethod
    def load_model(self):
        """
        :return: model
        """
        pass

    @abstractmethod
    def get_input_size_without_batch(self):
        pass

    @abstractmethod
    def format_output(self, outputs, input_features):
        pass

    @abstractmethod
    def pre_handle(self, *input_features):
        pass

    def infer(self, *input_features):
        with torch.no_grad():
            input_features_handled = self.pre_handle(*input_features)
            expect_input_shapes = self.get_input_size_without_batch()
            if type(expect_input_shapes) is list or type(expect_input_shapes) is tuple:
                for expect_input_shape, input_feature in zip(expect_input_shapes, input_features_handled):
                    is_same_size = self.compare_shape(expect_input_shape, input_feature)
                    if not is_same_size:
                        raise InputSizeException("input shape error")
            else:
                is_same_size = self.compare_shape(expect_input_shapes, input_features_handled)
                if not is_same_size:
                    raise InputSizeException("input shape error")
            per_size = self.config.batch_size
            if type(input_features_handled) is list or type(input_features_handled) is tuple:
                input_shape = input_features_handled[0].shape
            else:
                input_shape = input_features_handled.shape
            loop_size = int(input_shape[0] / per_size) + (0 if input_shape[0] % per_size == 0 else 1)
            outputs = []
            multi_outputs = []
            for index in range(loop_size):
                input_tensors = []
                if type(input_features_handled) is list or type(input_features_handled) is tuple:
                    for input_items in input_features_handled:
                        input_converted = input_items[
                                          index * per_size: min(input_shape[0], (index + 1) * per_size), ]
                        input_tensor = tensor_load_device(torch.from_numpy(np.array(input_converted)),
                                                          self.config.device)
                        input_tensors.append(input_tensor)
                else:
                    input_converted = input_features_handled[
                                      index * per_size: min(input_shape[0], (index + 1) * per_size), ]
                    input_tensors = tensor_load_device(torch.from_numpy(np.array(input_converted)), self.config.device)
                result = self.model(*tuple(input_tensors))
                if type(result) is list or type(result) is tuple:
                    if index == 0:
                        for result_index in range(len(result)):
                            multi_outputs.append([])
                    for result_item, result_index in zip(result, range(len(result))):
                        multi_outputs[result_index].append(result_item)
                else:
                    outputs.append(result)
            if len(outputs) > 0:
                outputs_cat = torch.cat(outputs, dim=0)
            else:
                outputs_cat = []
                for multi_output in multi_outputs:
                    outputs_cat.append(torch.cat(multi_output, dim=0))
            dic = {'outputs': outputs_cat, 'input_features': input_features}
            return self.format_output(**dic)

    @staticmethod
    def compare_shape(expect_input_shape, input_feature):
        input_shape = input_feature.shape
        shape_len = len(input_shape)
        is_same_size = True
        for index in range(shape_len - 1):
            is_same_size = is_same_size & (input_shape[index + 1] == expect_input_shape[index])
        return is_same_size



import numpy as np


def sigmoid(z):
    return 1 / (1 + np.exp(-z))


def flip180(arr):
    new_arr = arr.reshape(arr.size)
    new_arr = new_arr[::-1]
    new_arr = new_arr.reshape(arr.shape)
    return new_arr


def flip90_left(arr):
    new_arr = np.transpose(arr)
    new_arr = new_arr[::-1]
    return new_arr


def flip90_right(arr):
    new_arr = arr.reshape(arr.size)
    new_arr = new_arr[::-1]
    new_arr = new_arr.reshape(arr.shape)
    new_arr = np.transpose(new_arr)[::-1]
    return new_arr



import collections

from torch import nn


class TextCNNConv(nn.Module):
    """
    Convolution Block
    """

    def __init__(self, in_ch, out_ch, kernel_size):
        super(TextCNNConv, self).__init__()
        self.conv = nn.Sequential(collections.OrderedDict([
            ('text_cnn_conv_1', nn.Conv1d(in_ch, out_ch, kernel_size=kernel_size, bias=True)),
            ('text_cnn_relu_1', nn.ReLU(inplace=True)),
        ]))

    def forward(self, x):
        _out = self.conv(x)
        return _out



# !/home/wcg/tools/local/anaconda3/bin/python
# coding=utf8
import re


def crd_reduction_common(str_with_special, entry_list, special_character, text_process):
    txt_with_special_string = str_with_special
    special_characters = special_character.split("|")

    txt_with_special_string_list = []
    for i in range(len(txt_with_special_string)):
        if txt_with_special_string[i] in special_characters:
            txt_with_special_string_list.append('0')
        else:
            txt_with_special_string_list.append('1')

    for i in range(len(entry_list)):
        start = int(entry_list[i]['start'])
        word = entry_list[i]["word"]
        # word = entry_list[i]["value"]
        start_new = get_whithout_special_crd(txt_with_special_string_list, start)
        if text_process:
            word_new = text_process(word)
        else:
            word_new = re.sub(special_character, "", word)
        end_new = start_new + len(word_new)
        entry_list[i]['start'] = start_new
        entry_list[i]['end'] = end_new
        entry_list[i]["word"] = word_new
        # entry_list[i]["value"] = word_new
    return entry_list


def get_whithout_special_crd(txt_withspecialstring_list, num):
    crd_reduc = 0
    for i in range(num):
        crd_reduc = crd_reduc + int(txt_withspecialstring_list[i])
    return crd_reduc



import re
import datetime


def start_preprocess_bychar(s):
    s1 = ""
    for char in s:
        char_new = re.sub("：", ":", re.sub("＆", '&', re.sub('）', ')', re.sub('（', '(', str(char).upper()))))
        if len(char_new) == len(char):
            s1 += char_new
        elif len(char_new) > len(char):
            s1 += char_new[0]
        else:
            s1 += char
    return s1


def white_part_map(str_withspace_raw, text_preprocess):
    if len(str_withspace_raw)>0:
        str_no_space = text_preprocess(str_withspace_raw)
        maps = {0: 0}
        # print("str_withsapce_raw", str_withspace_raw)
        # print("str_no_space", str_no_space)
        if start_preprocess_bychar(str_withspace_raw[0]) == str_no_space[0]:
            start_same = True
            str_index = 1
        else:
            start_same = False
            str_index = 0
        to_end = False
        for i in range(1, len(str_withspace_raw)):
            # print(str_withspace_raw[i], str_no_space[str_index])
            if start_preprocess_bychar(str_withspace_raw[i]) == str_no_space[str_index] and not to_end:
                if start_same == False:
                    maps[i] = maps[i - 1]
                    start_same = True
                else:
                    maps[i] = maps[i - 1] + 1
                if str_index < len(str_no_space) - 1:
                    str_index += 1
                else:
                    to_end = True
            else:
                maps[i] = maps[i - 1]
            # if i < 20:
            #     print(str_withspace_raw[i], str_no_space[str_index], i, maps[i])
            # else:
            #     exit()
        # if len(str_withspace_raw) == len(str_no_space):
        #     print("-------same length----")
        if len(str_withspace_raw) < len(str_no_space):
            maps = {}
            for i in range(len(str_withspace_raw)):
                maps[i]=i
        else:
            assert maps[len(str_withspace_raw) - 1] == len(str_no_space) - 1
        return maps
    else:
        return {}


def space_reduction_common(str_withspace_raw, str_part, start_withoutspace, end_withoutspace, text_preprocess):
    start = datetime.datetime.now()
    maps = white_part_map(str_withspace_raw, text_preprocess)
    with_id = list(maps.keys())
    without_id = list(maps.values())
    if end_withoutspace == max(without_id) + 1:
        start_without_id = without_id.index(start_withoutspace)
        end_without_id = len(str_withspace_raw) - 1
    else:
        start_without_id = without_id.index(start_withoutspace)
        end_without_id = without_id.index(end_withoutspace - 1) + 1
    start_with_id = with_id[start_without_id]
    end_with_id = with_id[end_without_id]
    if end_with_id == len(str_withspace_raw) - 1:
        end_with_id = len(str_withspace_raw)
    new_word = str_withspace_raw[start_with_id:end_with_id]
    return new_word, start_with_id, end_with_id


def space_reduction_common_cc(str_withspace_raw, maps, start_withoutspace, end_withoutspace, text_preprocess):
    with_id = list(maps.keys())
    without_id = list(maps.values())
    if end_withoutspace == max(without_id) + 1:
        start_without_id = without_id.index(start_withoutspace)
        end_without_id = len(str_withspace_raw) - 1
    else:
        start_without_id = without_id.index(start_withoutspace)
        end_without_id = without_id.index(end_withoutspace - 1) + 1
    start_with_id = with_id[start_without_id]
    end_with_id = with_id[end_without_id]
    if end_with_id == len(str_withspace_raw) - 1:
        end_with_id = len(str_withspace_raw)
    new_word = str_withspace_raw[start_with_id:end_with_id]
    return new_word, start_with_id, end_with_id


def list_space_reduction(str_withspace, entities_list, text_preprocess):
    if len(entities_list) == 0:
        return []
    else:
        maps = white_part_map(str_withspace, text_preprocess)
        for i in range(len(entities_list)):
            entity = entities_list[i]
            str_part = entity["word"]
            start_withoutspace = entity['start']
            end_withoutspace = entity["end"]
            entity['word'], entity['start'], entity['end'] = space_reduction_common_cc(str_withspace, maps,
                                                                                       start_withoutspace,
                                                                                       end_withoutspace,
                                                                                       text_preprocess)
            entities_list[i] = entity
        return entities_list



from datetime import datetime


class NTLogger:
    def __init__(self, context, verbose):
        self.context = context
        self.verbose = verbose

    def info(self, msg, **kwargs):
        if self.verbose:
            print('I:%s:%s:%s' % (str(datetime.now()), self.context, msg), flush=True)

    def debug(self, msg, **kwargs):
        if self.verbose:
            print('D:%s:%s:%s' % (str(datetime.now()), self.context, msg), flush=True)

    def error(self, msg, **kwargs):
        print('E:%s:%s:%s' % (str(datetime.now()), self.context, msg), flush=True)

    def warning(self, msg, **kwargs):
        print('W:%s:%s:%s' % (str(datetime.now()), self.context, msg), flush=True)



import numpy as np
import cv2
from PIL import Image


def load_image(path, resize_height, resize_width, gray, normalization):
    image = Image.open(path)
    input_image = image.resize((resize_width, resize_height))
    if gray:
        input_image = np.array(input_image.convert('L'))
    else:
        input_image = np.array(input_image.convert('RGB'))
    if normalization:
        input_image = input_image / 255
    return input_image


def draw_rectangle(draw, points, color, width):
    draw.line([(points[0], points[1]), (points[2], points[1])], fill=color, width=width)
    draw.line([(points[2], points[1]), (points[2], points[3])], fill=color, width=width)
    draw.line([(points[2], points[3]), (points[0], points[3])], fill=color, width=width)
    draw.line([(points[0], points[3]), (points[0], points[1])], fill=color, width=width)
    return draw



import os

import cv2


def draw_block(image_binary_array, mutipage_texts_row_blocked, file_name, config):
    '''
    文档分块 将分块绘制到远图中，以判断分块效果
    :param image_binary_array: numpy.array 表示多个图片的binary信息 由 read_and_convert_file返回
    :param mutipage_texts_row_blocked: numpy.array 表示分块信息 由tests_to_blocks 返回
    :param file_name: str 文件名称
    :param config: BasicConfig 表config实例 传入为LocalConfig实例
   :return: Null
    '''
    assert len(image_binary_array) == len(mutipage_texts_row_blocked), "图片数要和页数相等"
    for i in range(len(image_binary_array)):
        image_binary = image_binary_array[i]
        texts_rows_blocked = mutipage_texts_row_blocked[i]
        img = cv2.imdecode(image_binary, cv2.IMREAD_COLOR)
        h, w, _ = img.shape
        for texts_rows in texts_rows_blocked:
            block_index = texts_rows["block_index"]
            text_contents = texts_rows["text_contexts"]
            if len(text_contents) <= 1:
                continue
            row_min = text_contents[0]
            row_max = text_contents[-1]
            row_min_left = row_min[0]
            row_max_right = row_max[-1]
            y_top_min = row_min_left.values()[1]
            y_bottom_max = row_max_right.values()[3]

            draw_x_min = 0
            draw_x_max = w
            draw_y_min = y_top_min
            draw_y_max = y_bottom_max
            cv2.rectangle(img, (draw_x_min, draw_y_min), (draw_x_max, draw_y_max), (255, 0, 0), 2)
            cv2.imwrite(os.path.join(config.convert_config.get_tmp_file_dir(),
                                     "draw_blocked_" + file_name + "_" + str(i) + ".jpg"), img)



import os

from yqn_common.str_util import ends_with


def get_absolute_path(p):
    if p.startswith('~'):
        p = os.path.expanduser(p)
    return os.path.abspath(p)


def ls(path='.', suffix=None):
    path = get_absolute_path(path)
    files = os.listdir(path)

    if suffix is None:
        return files

    filtered = []
    for f in files:
        if ends_with(f, suffix, ignore_case=True):
            filtered.append(f)

    return filtered



from distutils.core import setup

from setuptools import find_packages

# read the contents of the README
with open('README.md') as README_md:
    README = README_md.read()

if __name__ == '__main__':
    setup(
        name='yqn-pytorch-framework',
        version='0.1.25',
        author='James Guo',
        author_email="james.guo89@gmail.com",
        description='YQN Pytorch Framework',
        url='https://www.google.com',
        long_description=README,
        long_description_content_type="text/markdown",
        license='Proprietary',
        package_data={'yqn_cli': ['template/*/*', ]},
        packages=find_packages(exclude=['tests', '*.tests', '*.tests.*']),
        install_requires=['opencv_python>=4.2.0.32',
                          'numpy>=1.18.2',
                          'torch>=1.4.0',
                          'torchvision>=0.5.0',
                          'nvidia_ml_py3>=7.352.0',
                          'setuptools>=41.2.0',
                          'oss2>=2.9.1',
                          'Pillow>=6.2.0',
                          'jieba>=0.39',
                          'pynvml>=8.0.4',
                          'PyYAML>=5.3', ],
        entry_points={
            'console_scripts': [
                'yqn.module = yqn_cli.init_model:main'
            ],
        },
        keywords=' '.join([
            'pytorch', 'framework'
        ]),
        classifiers=[
            'Development Status :: 4 - Beta',
            'Environment :: Console',
            'Environment :: Web Environment',
            'Intended Audience :: End Users/Desktop',
            'Intended Audience :: Developers',
            'Intended Audience :: System Administrators',
            "License :: OSI Approved :: MIT License",
            'Operating System :: MacOS :: MacOS X',
            'Operating System :: Microsoft :: Windows',
            'Operating System :: POSIX',
            'Programming Language :: Python',
            'Topic :: Communications :: Email',
            'Topic :: Office/Business',
            'Topic :: Software Development :: Bug Tracking',
        ],
    )
